"""This module manages the saved PDF files and opens them if required.

This file is part of the physbiblio package.
"""
import sys
import six
import shutil
import traceback
if sys.version_info[0] < 3:
	from urllib2 import urlopen, HTTPError
else:
	from urllib.request import urlopen, HTTPError
import os.path as osp
import os
import subprocess

try:
	from physbiblio.config import pbConfig
	from physbiblio.errors import pBLogger
	from physbiblio.database import pBDB
except ImportError:
	print("Could not find physbiblio and its modules!")
	print(traceback.format_exc())
	raise


class LocalPDF():
	"""Class with functions to manage the PDF folder content
	and the stored material
	"""

	pdfDir = pbConfig.params["pdfFolder"]
	pdfApp = pbConfig.params["pdfApplication"]
	badFNameCharacters = r'\/:*?"<>|' + "'"

	def __init__(self):
		"""Init the class and set some default variables"""
		self.checkFolderExists()
		self.pdfDir = pbConfig.params["pdfFolder"]
		self.pdfApp = pbConfig.params["pdfApplication"]

	def checkFolderExists(self):
		"""Check if the PDF folder exists. If not, create it"""
		if not os.path.isdir(self.pdfDir):
			pBLogger.info("PDF folder missing: %s. Creating it."%self.pdfDir)
			os.makedirs(self.pdfDir)

	def badFName(self, filename):
		"""Clean the filename substituting the bad characters with '_'

		Parameters:
			filename (string): the string containing the file name

		Output:
			the cleaned filename
		"""
		newFilename = ""
		if not isinstance(filename, six.string_types):
			pBLogger.warning("Wrong filename type! %s"%filename)
			return ""
		for i in filename:
			if i not in self.badFNameCharacters:
				newFilename += i
			else:
				newFilename += "_"
		return newFilename

	def getFileDir(self, key):
		"""Obtain the name of the directory for a given entry.
		The name is cleaned and the absolute path is generated.

		Parameters:
			key (string): the bibtex key of the entry

		Output:
			the (cleaned) absolute path for the PDF files
				associated with the entry
		"""
		return osp.join(self.pdfDir, self.badFName(key))

	def getFilePath(self, key, fileType):
		"""Obtain the file path for a given file type (arxiv, doi, ...)
		of a given entry.
		It reads the field from the database in order
		to generate the file name.

		Parameters:
			key (string): the bibtex key of the entry
			fileType (string in
				["arxiv", "inspire", "isbn", "doi", "ads", "scholar"]):
				basically the field in the database
				from which the name must be taken

		Output:
			the (cleaned) absolute path for the PDF file
		"""
		if fileType not in ["arxiv", "inspire", "isbn",
				"doi", "ads", "scholar"]:
			pBLogger.warning("Required field does not exist or is not valid")
			return ""
		filename = pBDB.bibs.getField(key, fileType)
		if filename is None:
			pBLogger.warning("Impossible to get the type '%s' "%fileType
				+ "filename for entry %s"%key)
			return ""
		else:
			filename = self.badFName(filename)
			return osp.join(self.getFileDir(key), filename + '.pdf')

	def createFolder(self, key, noCheck=False):
		"""Create the PDF folder for a given entry.

		Parameters:
			key (string): the bibtex key of the entry
			noCheck (bool): True to avoid checking
				if the directory exists. Default False
		"""
		directory = self.getFileDir(key)
		if noCheck or not osp.exists(directory):
			os.makedirs(directory)

	def renameFolder(self, oldkey, newkey):
		"""Rename the PDF folder for a given entry
		for which the bibtex key has been changed.

		Parameters:
			oldkey (string): the old bibtex key of the entry
			newkey (string): the new bibtex key of the entry
		"""
		olddir = self.getFileDir(oldkey)
		if osp.exists(olddir):
			newdir = self.getFileDir(newkey)
			pBLogger.info("Renaming %s to %s"%(olddir, newdir))
			return shutil.move(olddir, newdir)
		else:
			return False

	def copyNewFile(self,
			key,
			origFileName,
			fileType=None,
			customName=None):
		"""Copy a file in any place of the filesystem
		into the directory corresponding to the given entry

		Parameters:
			key (string): the bibtex key of the entry
			origFileName (string): the name of the file to be copied
			fileType (string in
				["arxiv", "inspire", "isbn", "doi", "ads", "scholar"]):
				basically the field in the database
				from which the name must be taken.
				`customName` can be given instead.
			customName (string):
				a completely independent name for the file.
				It may be given instead of fileType.
		"""
		if not osp.exists(self.getFileDir(key)):
			self.createFolder(key, True)
		if customName is not None:
			newFileName = osp.join(self.getFileDir(key), customName)
		elif fileType is not None:
			newFileName = self.getFilePath(key, fileType)
		else:
			pBLogger.warning("You should supply a fileType "
				+ "('doi' or 'arxiv') or a customName!")
			return False
		try:
			shutil.copy2(origFileName, newFileName)
			pBLogger.info("%s copied to %s"%(origFileName, newFileName))
			return True
		except:
			pBLogger.exception("Impossible to copy %s to %s"%(
				origFileName, newFileName))
			return False

	def copyToDir(self, outFolder, key, fileType=None, customName=None):
		"""Copy a file from the directory corresponding
		to the given entry to the a directory in the filesystem.

		Parameters:
			outFolder (string): the name of the destination folder
			key (string): the bibtex key of the entry
			fileType (string in
				["arxiv", "inspire", "isbn", "doi", "ads", "scholar"]):
				basically the field in the database from which
				the name must be taken. customName can be given instead.
			customName (string):
				a completely independent name for the file.
				It may be given instead of fileType.
		"""
		if customName is not None:
			origFile = osp.join(self.getFileDir(key), customName)
		elif fileType is not None:
			origFile = self.getFilePath(key, fileType)
		else:
			pBLogger.warning("You should supply a fileType "
				+ "('doi' or 'arxiv') or a customName!")
			return False
		try:
			shutil.copy2(origFile, outFolder)
			pBLogger.info("%s copied to %s"%(origFile, outFolder))
			return True
		except:
			pBLogger.exception("Impossible to copy %s to %s"%(
				origFile, outFolder))
			return False

	def downloadArxiv(self, key, force=False):
		"""Download the PDF file from arXiv for a given entry
		and save it in the proper folder.

		Parameters:
			key (string): the bibtex key of the entry
			force (boolean, optional, default False):
				force the replacement of the file if it already exists.
				If False and there is a PDF with the same name,
				no action is performed.
		"""
		filename = self.getFilePath(key, "arxiv")
		if osp.exists(filename) and not force:
			pBLogger.info(
				"There is already a pdf and overwrite not requested.")
			return True
		try:
			self.createFolder(key)
			url = pBDB.bibs.getArxivUrl(key, 'pdf')
			if url is False:
				pBLogger.warning("Invalid arXiv PDF url for '%s',"%key
					+ " probably the field is empty.")
				return False
			pBLogger.info("Downloading arXiv PDF from %s"%url)
			response = urlopen(url)
			with open(filename, 'wb') as newF:
				newF.write(response.read())
			pBLogger.info("File saved to %s"%filename)
		except HTTPError:
			pBLogger.exception("ArXiv PDF for '%s' not found "%key
				+ "(404 error on url: %s)"%url)
			return False
		except:
			pBLogger.exception(
				"Impossible to download the arXiv PDF for '%s'"%key)
			return False
		else:
			return os.path.exists(filename)

	def openFile(self,
			key,
			arg=None,
			fileType=None,
			fileNum=None,
			fileName=None):
		"""Open a PDF file in an external application
		(if defined in the configuration).
		Does nothing if there is self.pdfApp is empty.

		Parameters:
			key (string): the bibtex key of the entry
			at least one of the following (they are considered in this order):
				arg: used to guess the fileType, fileNum or fileName
				fileType (string in
					["arxiv", "inspire", "isbn", "doi", "ads", "scholar"]):
					basically the field in the database
					from which the name must be taken
				fileNum: the number of the desired PDF
					in the list of existing PDF files for the considered entry
				filaName: the specific file name
		"""
		try:
			if arg == "arxiv" or arg == "doi":
				fileType = arg
			elif float(arg).is_integer():
				fileNum = int(arg)
		except:
			if arg is not None:
				fileName = arg
		try:
			if fileType is not None:
				fName = self.getFilePath(key, fileType)
			elif fileNum is not None:
				fName = self.getExisting(key, True)[fileNum]
			elif fileName is not None:
				fName = osp.join(self.getFileDir(key), fileName)
			else:
				pBLogger.warning("Invalid selection. "
					+ "One among fileType, fileNum or fileName must be given!")
				return
			if self.pdfApp != "":
				pBLogger.info("Opening '%s'..."%fName)
				subprocess.Popen([self.pdfApp, fName],
					stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
		except:
			pBLogger.exception("Opening PDF for '%s' failed!"%key)

	def checkFile(self, key, fileType):
		"""Check if a file of a given type (arxiv, doi, ...)
		and a given entry exists.

		Parameters:
			key (string): the bibtex key of the entry
			fileType (string in
				["arxiv", "inspire", "isbn", "doi", "ads", "scholar"]):
				basically the field in the database
				from which the name must be taken

		Output:
			boolean (if the arguments are valid) or None (if fileType is bad)
		"""
		if fileType not in ["arxiv", "inspire", "isbn",
				"doi", "ads", "scholar"]:
			pBLogger.warning("Invalid argument to checkFile!")
			return
		return os.path.isfile(self.getFilePath(key, fileType))

	def removeFile(self, key, fileType, fileName=None):
		"""Delete a PDF file.

		Parameters:
			key (string): the bibtex key of the entry
			fileType (any string if fileName is not None or string in
				["arxiv", "inspire", "isbn", "doi", "ads", "scholar"]):
				basically the field in the database
				from which the name must be taken
			fileName (default None): specify the file name instead
				of computing it from the database information

		Output:
			True if successful, False if exception occurred
		"""
		if fileName is None:
			fileName = self.getFilePath(key, fileType)
		try:
			os.remove(fileName)
		except OSError:
			pBLogger.exception("Impossible to remove file: %s"%fileName)
			return False
		pBLogger.info("File %s removed"%fileName)
		return True

	def getExisting(self, key, fullPath=False):
		"""Obtain the list of existing files for a given entry.

		Parameters:
			key (string): the bibtex key of the entry
			fullPath (boolean, default False):
				return the list with absolute paths

		Output:
			a list, possibly empty
		"""
		fileDir = self.getFileDir(key)
		try:
			dircontent = os.listdir(fileDir)
		except Exception:
			return []
		try:
			files = [e for e in dircontent if osp.isfile(osp.join(fileDir, e))]
		except (FileNotFoundError, IOError):
			pBLogger.exception("Error in listing the files")
			return []
		if fullPath:
			files = [osp.join(fileDir, e) for e in files]
		return files

	def printExisting(self, key, fullPath=False):
		"""Print the list of existing files for a given entry,
		using self.getExisting to get it.
		Same parameters as self.getExisting.
		"""
		pBLogger.info("Listing file for entry '%s', located in %s:"%(
			key, self.getFileDir(key)))
		for i,e in enumerate(self.getExisting(key, fullPath=fullPath)):
			pBLogger.info("%2d: %s"%(i, e))

	def printAllExisting(self, entries=None, fullPath=False):
		"""Print the complete list of all the existing PDF files
		in the PDF folder, given all the entries in the database
		or a specific subset

		Parameters:
			entries (list, optional): the list of bibtex keys to consider.
				If None, get all the database content
			fullPath (boolean, default False):
				print the list with absolute paths
		"""
		iterator = entries
		if entries is None:
			pBDB.bibs.fetchAll(orderBy="firstdate",
				saveQuery=False, doFetch=False)
			iterator = pBDB.bibs.fetchCursor()
		for e in iterator:
			exist = self.getExisting(e["bibkey"], fullPath=fullPath)
			if len(exist) > 0:
				pBLogger.info("%30s: [%s]"%(e["bibkey"], "] [".join(exist)))

	def removeSparePDFFolders(self):
		"""Scans the PDF folder in order to find single unassociated
		directories (their name is not matched by any database entry).
		The unassociated directories are removed without asking any
		additional confirmation. Be careful!
		"""
		pBDB.bibs.fetchAll(doFetch=False)
		folders = os.listdir(self.pdfDir)
		for e in pBDB.bibs.fetchCursor():
			k = e["bibkey"]
			cleaned = self.badFName(k)
			if cleaned in folders:
				del folders[folders.index(cleaned)]
		if len(folders) > 0:
			pBLogger.info("Spare PDF folders found: %d\n%s\n"%(
				len(folders), folders)
				+ "They will be removed now.")
			for f in folders:
				shutil.rmtree(osp.join(self.pdfDir, f))
			pBLogger.info("Done!")
		else:
			pBLogger.warning("Nothing found.")

	def mergePDFFolders(self, oldkey, newkey):
		"""Copy the PDF files that exist in one folder to a new one

		Parameters:
			oldkey: the old bibtex key
			newkey: the new bibtex key
		"""
		oldPDFs = self.getExisting(oldkey, fullPath=True)
		outFolder = self.getFileDir(newkey)
		self.createFolder(newkey)
		for o in oldPDFs:
			try:
				shutil.copy2(o, outFolder)
				pBLogger.info("%s copied to %s"%(o, outFolder))
			except:
				pBLogger.exception(
					"Impossible to copy %s to %s"%(o, outFolder))

	def numberOfFiles(self, folder):
		"""Get the total number of files inside the given folder

		Parameters:
			folder: the path of the folder to scan

		Output:
			the number of files
		"""
		total_number = 0
		if six.PY2:
			error_class = OSError
		else:
			error_class = FileNotFoundError
		if sys.version_info[0] < 3:
			try:
				dirlist = os.listdir(folder)
			except error_class:
				dirlist = []
			for item in dirlist:
				itempath = os.path.join(folder, item)
				if os.path.isfile(itempath):
					total_number += 1
				elif os.path.isdir(itempath):
					total_number += self.numberOfFiles(itempath)
		else:
			for dirpath, dirnames, filenames in os.walk(folder):
				for f in filenames:
					fp = os.path.join(dirpath, f)
					if os.path.isfile(fp):
						total_number += 1
		return total_number

	def dirSize(self, folder, dirs=True):
		"""Get the size of a single directory and its content

		Parameters:
			folder: the path of the folder to scan
			dirs (default True): if True, include the size of folders

		Output:
			the size in bytes
		"""
		if dirs:
			if six.PY2:
				error_class = OSError
			else:
				error_class = FileNotFoundError
			try:
				total_size = os.path.getsize(folder)
			except error_class:
				pBLogger.error(
					"PDF folder is missing: %s. Creating it."%folder)
				os.makedirs(folder)
				return os.path.getsize(folder)
		else:
			total_size = 0
		if sys.version_info[0] < 3:
			for item in os.listdir(folder):
				itempath = os.path.join(folder, item)
				if os.path.isfile(itempath):
					total_size += os.path.getsize(itempath)
				elif os.path.isdir(itempath):
					total_size += self.dirSize(itempath, dirs=dirs)
		else:
			for dirpath, dirnames, filenames in os.walk(folder):
				if dirs:
					for d in dirnames:
						total_size += os.path.getsize(
							os.path.join(dirpath, d))
				for f in filenames:
					fp = os.path.join(dirpath, f)
					if os.path.isfile(fp):
						total_size += os.path.getsize(fp)
		return total_size

	def getSizeWUnits(self, size, units="MB", fmt="%.2f"):
		"""Print a size obtained with `self.dirSize`

		Parameters:
			size:
			units (default "MB"):
			format (default "%.2f"): the format

		Output:
			the string with the size in the appropriate units
		"""
		allowedUnits = ["B", "KB", "MB", "GB", "TB", "PB"]
		try:
			exponent = allowedUnits.index(units.upper())
			unitsU = units.upper()
		except ValueError:
			pBLogger.warning("Invalid units. Changing to 'MB'.")
			exponent = 2
			unitsU = "MB"
		try:
			sizeWUnits = size / 1024.**(exponent)
		except TypeError:
			pBLogger.warning("Invalid size. It must be a number!")
			return "nan"
		try:
			return fmt%sizeWUnits + unitsU
		except (TypeError, ValueError):
			pBLogger.warning("Invalid format. Using '%.2f'")
			return "%.2f"%sizeWUnits + unitsU


pBPDF = LocalPDF()



#!/usr/bin/env python
"""Test file for the physbiblio.config module.

This file is part of the physbiblio package.
"""
import sys
import traceback
import os

if sys.version_info[0] < 3:
	import unittest2 as unittest
	from mock import patch
else:
	import unittest
	from unittest.mock import patch

try:
	from physbiblio.setuptests import *
	from physbiblio.databaseCore import PhysBiblioDBCore
	from physbiblio.config import \
		pbConfig, ConfigVars, config_defaults, ConfigurationDB, GlobalDB
except ImportError:
	print("Could not find physbiblio and its modules!")
	raise
except Exception:
	print(traceback.format_exc())

tempOldCfgName = os.path.join(pbConfig.dataPath, "tests_%s.cfg"%today_ymd)
tempCfgName = os.path.join(pbConfig.dataPath, "tests_cfg_%s.db"%today_ymd)
tempCfgName1 = os.path.join(pbConfig.dataPath, "tests_cfg1_%s.db"%today_ymd)
tempProfName = os.path.join(pbConfig.dataPath, "tests_prof_%s.db"%today_ymd)

class TestConfigMethods(unittest.TestCase):
	"""Tests for methods in physbiblio.config"""

	@patch('sys.stdout', new_callable=StringIO)
	def assert_in_stdout_multi(self, function, expected_output, mock_stdout):
		"""Catch and if test stdout of the function contains a string"""
		pBErrorManager.tempHandler(sys.stdout, format='%(message)s')
		function()
		pBErrorManager.rmTempHandler()
		for e in expected_output:
			self.assertIn(e, mock_stdout.getvalue())

	def test_config(self):
		"""Test config methods for config management"""
		if os.path.exists(tempCfgName):
			os.remove(tempCfgName)
		newConfParamsDict = dict(config_defaults)
		with patch("physbiblio.config.ConfigVars.readProfiles",
				return_value=("tmp", {"tmp": {"db": tempCfgName, "d":""}},
				["tmp"]), autospec=True) as _mock_readprof:
			self.assertFalse(os.path.exists(tempCfgName))
			tempPbConfig = ConfigVars()
			tempPbConfig.prepareLogger("physbibliotestlog")
			tempDb = PhysBiblioDBCore(tempCfgName,
				tempPbConfig.logger, info=False)
			configDb = ConfigurationDB(tempDb)

			self.assertTrue(os.path.exists(tempCfgName))
			_mock_readprof.assert_called_once_with(tempPbConfig)
			self.assertEqual(len(configDb.getAll()), 0)

			tempPbConfig1 = ConfigVars()
			tempPbConfig1.prepareLogger("physbibliotestlog")
			self.assertEqual(tempPbConfig1.params, newConfParamsDict)
			tempPbConfig1.params["logFileName"] = "otherfilename"
			tempPbConfig1.params["timeoutWebSearch"] = 10.
			tempPbConfig1.params["askBeforeExit"] = True
			tempPbConfig1.params["maxAuthorNames"] = 5
			tempPbConfig1.params["defaultCategories"] = [1]
			with open(tempOldCfgName, "w") as f:
				f.write("logFileName = otherfilename\n"
					+ "timeoutWebSearch = 10.0\n"
					+ "askBeforeExit = True\n"
					+ "maxAuthorNames = 5\n"
					+ "defaultCategories = [1]\n")
			tempPbConfig = ConfigVars()
			tempPbConfig.prepareLogger("physbibliotestlog")
			tempPbConfig.configMainFile = tempOldCfgName
			tempPbConfig.oldReadConfigFile()
			self.assertEqual(tempPbConfig.params, tempPbConfig1.params)

			with patch.dict(os.environ,
						{"PHYSBIBLIO_CONFIG": "", "PHYSBIBLIO_DATA": ""},
						clear=False),\
					patch("physbiblio.config.GlobalDB",
						return_value=tempPbConfig.globalDb) as _c:
				tempPbConfig1 = ConfigVars()
				self.assertNotEqual(tempPbConfig1.configPath, ".")
				self.assertNotEqual(tempPbConfig1.dataPath, ".")
			with patch.dict(os.environ,
						{"PHYSBIBLIO_CONFIG": ".", "PHYSBIBLIO_DATA": "."},
						clear=False),\
					patch("physbiblio.config.GlobalDB",
						return_value=tempPbConfig.globalDb) as _c:
				tempPbConfig1 = ConfigVars()
				self.assertEqual(tempPbConfig1.configPath, ".")
				self.assertEqual(tempPbConfig1.dataPath, ".")
		os.remove(tempCfgName)
		os.remove(tempOldCfgName)

	def test_profiles(self):
		"""Test config methods for profiles management"""
		if os.path.exists(tempProfName):
			os.remove(tempProfName)
		tempPbConfig = ConfigVars(tempProfName)
		tempPbConfig.prepareLogger("physbibliotestlog")
		tempPbConfig.reloadProfiles()
		self.assertEqual(tempPbConfig.defaultProfileName, "default")
		self.assertEqual(tempPbConfig.profiles,
			{u"default": {
				"d": u"",
				"f": u"",
				"n": u"default",
				"db": str(os.path.join(pbConfig.dataPath,
				config_defaults["mainDatabaseName"].replace("PBDATA", "")))}})
		self.assertEqual(tempPbConfig.profileOrder, ["default"])

		tempProfName1 = os.path.join(pbConfig.dataPath,
			"tests_prof1_%s.db"%today_ymd)
		self.assertTrue(tempPbConfig.globalDb.createProfile("temp",
			"none", tempProfName1))
		self.assertTrue(tempPbConfig.globalDb.setDefaultProfile("temp"))
		self.assertEqual(tempPbConfig.defaultProfileName, "default")
		self.assertEqual([e["n"] for e in tempPbConfig.profiles.values()],
			["default"])
		self.assertEqual(tempPbConfig.profileOrder, ["default"])
		self.assertEqual(tempPbConfig.currentProfileName, "default")
		self.assertEqual(tempPbConfig.currentDatabase,
			str(os.path.join(pbConfig.dataPath,
			config_defaults["mainDatabaseName"].replace("PBDATA", ""))))
		tempParams = tempPbConfig.params

		default, profiles, ordered = tempPbConfig.readProfiles()
		self.assertEqual(default, "temp")
		self.assertEqual(profiles,
			{u"default": {
				"d": u"",
				"f": u"",
				"n": u"default",
				"db": str(os.path.join(pbConfig.dataPath,
				config_defaults["mainDatabaseName"].replace("PBDATA", "")))},
			u"temp": {
				"d": u"none",
				"f": u"",
				"n": u"temp",
				"db": tempProfName1}})
		self.assertEqual(ordered, ["default", "temp"])
		tempPbConfig.reloadProfiles()
		self.assertEqual(tempPbConfig.params, config_defaults)

		tempProfName2 = tempProfName1.replace("prof1", "prof2")
		self.assertTrue(tempPbConfig.globalDb.createProfile("other",
			"none1", tempProfName2))
		tempPbConfig.reloadProfiles()
		self.assertEqual(tempPbConfig.defaultProfileName, "temp")
		self.assertEqual(sorted(
			[e["n"] for e in tempPbConfig.profiles.values()]),
			["default", "other", "temp"])
		self.assertEqual(tempPbConfig.profileOrder,
			["default", "other", "temp"])
		self.assertEqual(tempPbConfig.currentProfileName, "temp")
		self.assertEqual(tempPbConfig.currentProfile,
			tempPbConfig.profiles["temp"])
		self.assertEqual(tempPbConfig.currentDatabase, tempProfName1)

		tempPbConfig.reInit("other")
		self.assertEqual(tempPbConfig.defaultProfileName, "temp")
		self.assertEqual(sorted(
			[e["n"] for e in tempPbConfig.profiles.values()]),
			["default", "other", "temp"])
		self.assertEqual(tempPbConfig.profileOrder,
			["default", "other", "temp"])
		self.assertEqual(tempPbConfig.currentProfileName, "other")
		self.assertEqual(tempPbConfig.currentProfile,
			tempPbConfig.profiles["other"])
		self.assertEqual(tempPbConfig.currentDatabase, tempProfName2)

		tempPbConfig.reInit("other", tempPbConfig.profiles["temp"])
		self.assertEqual(tempPbConfig.currentProfileName, "other")
		self.assertEqual(tempPbConfig.currentProfile,
			tempPbConfig.profiles["temp"])
		self.assertEqual(tempPbConfig.currentDatabase, tempProfName1)

		tempPbConfig.reInit("temp", tempPbConfig.profiles["temp"])
		self.assertEqual(tempPbConfig.currentProfileName, "temp")
		self.assertEqual(tempPbConfig.currentProfile,
			tempPbConfig.profiles["temp"])
		self.assertEqual(tempPbConfig.currentDatabase, tempProfName1)

		tempPbConfig.reInit("nonexistent")
		self.assertEqual(tempPbConfig.currentProfileName, "temp")
		self.assertEqual(tempPbConfig.currentProfile,
			tempPbConfig.profiles["temp"])
		self.assertEqual(tempPbConfig.currentDatabase, tempProfName1)
		self.assertEqual(tempPbConfig.params, tempParams)

		tempDb = PhysBiblioDBCore(tempProfName1,
			tempPbConfig.logger, info=False)
		configDb = ConfigurationDB(tempDb)
		configDb.insert("defaultCategories", "[0")
		configDb.commit()

		tempPbConfig.readConfig()
		self.assertEqual(tempPbConfig.params, tempParams)
		self.assert_in_stdout_multi(tempPbConfig.readConfig,
			["Failed in reading parameter 'defaultCategories'."])

		configDb.insert("loggingLevel", "4")
		configDb.insert("pdfFolder", "/nonexistent/")
		configDb.insert("askBeforeExit", "True")
		configDb.insert("timeoutWebSearch", "20.")
		configDb.commit()
		tempParams["loggingLevel"] = 4
		tempParams["pdfFolder"] = u"/nonexistent/"
		tempParams["askBeforeExit"] = True
		tempParams["timeoutWebSearch"] = 20.
		tempPbConfig.readConfig()
		self.assertEqual(tempPbConfig.params, tempParams)
		self.assert_in_stdout_multi(tempPbConfig.readConfig,
			["Failed in reading parameter 'defaultCategories'."])

		configDb.insert("defaultCategories", "[0]")
		configDb.commit()
		tempParams["defaultCategories"] = [0]
		tempPbConfig.readConfig()
		self.assertEqual(tempPbConfig.params, tempParams)

		tempDb.closeDB(info=False)
		if os.path.exists(tempProfName1):
			os.remove(tempProfName1)
		if os.path.exists(tempProfName2):
			os.remove(tempProfName2)

	def test_searches(self):
		"""Test config methods for profiles management"""
		if os.path.exists(tempProfName):
			os.remove(tempProfName)
		tempPbConfig = ConfigVars(tempProfName)
		self.assertEqual(tempPbConfig.globalDb.countSearches(), 0)

		self.assertTrue(tempPbConfig.globalDb.insertSearch(
			"testname", 123, {"field": "value"},
			["a", "b", "c", "d", "e"], True, True, 21, 321))
		self.assertEqual(tempPbConfig.globalDb.countSearches(), 1)
		search = tempPbConfig.globalDb.getSearchByID(1)[0]
		self.assertEqual(search["name"], "testname")
		self.assertEqual(search["count"], 123)
		self.assertEqual(search["manual"], 1)
		self.assertEqual(search["isReplace"], 1)
		self.assertEqual(search["limitNum"], 21)
		self.assertEqual(search["offsetNum"], 321)
		self.assertEqual(search["searchDict"], "{'field': 'value'}")
		self.assertEqual(search["replaceFIelds"], "['a', 'b', 'c', 'd', 'e']")
		searchN = tempPbConfig.globalDb.getSearchByName("testname")[0]
		self.assertEqual(search, searchN)

		self.assertTrue(tempPbConfig.globalDb.insertSearch(
			"test1", 2, {}, [], True, False))
		self.assertTrue(tempPbConfig.globalDb.insertSearch(
			"test2", 1, {}, [], True, True))
		self.assertTrue(tempPbConfig.globalDb.insertSearch(
			"test3", 3, {}, [], False, True))
		self.assertTrue(tempPbConfig.globalDb.insertSearch(
			"test4", 0, {}, [], False, True))
		self.assertTrue(tempPbConfig.globalDb.insertSearch(
			"test5", 122, {}, [], False, True))
		self.assertEqual(tempPbConfig.globalDb.countSearches(), 6)
		self.assertEqual(
			[e["name"] for e in tempPbConfig.globalDb.getAllSearches()],
			["test4", "test2", "test1", "test3", "test5", "testname"])
		self.assertEqual(
			[e["name"] for e in tempPbConfig.globalDb.getSearchList()],
			[])
		self.assertEqual(
			[e["name"] for e in tempPbConfig.globalDb.getSearchList(True)],
			["test1"])
		self.assertEqual(
			[e["name"] for e in \
				tempPbConfig.globalDb.getSearchList(False, True)],
			["test4", "test3", "test5"])
		self.assertEqual(
			[e["name"] for e in \
				tempPbConfig.globalDb.getSearchList(True, True)],
			["test2", "testname"])
		with patch("logging.Logger.warning") as _e:
			self.assertFalse(
				tempPbConfig.globalDb.updateSearchField(1, "abc", "ab"))
			_e.assert_called_once_with(
				'Empty value or field not in the following list: '
				+ '[searchDict, replaceFields, name, limitNum, offsetNum]')
			_e.reset_mock()
			self.assertFalse(
				tempPbConfig.globalDb.updateSearchField(1, "searchDict", ""))
			_e.assert_called_once_with(
				'Empty value or field not in the following list: '
				+ '[searchDict, replaceFields, name, limitNum, offsetNum]')
			_e.reset_mock()
			self.assertFalse(
				tempPbConfig.globalDb.updateSearchField(
					1, "replaceFields", None))
			_e.assert_called_once_with(
				'Empty value or field not in the following list: '
				+ '[searchDict, replaceFields, name, limitNum, offsetNum]')
			_e.reset_mock()
			self.assertTrue(
				tempPbConfig.globalDb.updateSearchField(1, "searchDict", "ab"))
			_e.assert_not_called()
			self.assertTrue(
				tempPbConfig.globalDb.updateSearchField(
					1, "replaceFields", "cd"))
			_e.assert_not_called()
			self.assertTrue(
				tempPbConfig.globalDb.updateSearchField(1, "name", "BA"))
			_e.assert_not_called()
			self.assertTrue(
				tempPbConfig.globalDb.updateSearchField(1, "limitNum", "91"))
			_e.assert_not_called()
			self.assertTrue(
				tempPbConfig.globalDb.updateSearchField(1, "offsetNum", "19"))
			_e.assert_not_called()
		search = tempPbConfig.globalDb.getSearchByID(1)[0]
		self.assertEqual(search["searchDict"], "ab")
		self.assertEqual(search["name"], "BA")
		self.assertEqual(search["replaceFIelds"], "cd")
		self.assertEqual(search["limitNum"], 91)
		self.assertEqual(search["offsetNum"], 19)

		with patch.dict(pbConfig.params,
				{"maxSavedSearches": 5},
				clear=False):
			self.assertTrue(tempPbConfig.globalDb.updateSearchOrder(True))
			self.assertEqual([e["name"] for e in \
				tempPbConfig.globalDb.getSearchList(False, True)],
				["test4", "test3"])
			self.assertTrue(tempPbConfig.globalDb.updateSearchOrder(True))
			self.assertEqual([e["name"] for e in \
				tempPbConfig.globalDb.getSearchList(False, True)],
				["test4"])
			self.assertTrue(tempPbConfig.globalDb.updateSearchOrder(True))
			self.assertEqual([e["name"] for e in \
				tempPbConfig.globalDb.getAllSearches()],
				["test2", "test1", "test4", "BA"])

			self.assertTrue(tempPbConfig.globalDb.insertSearch(
				"testA", 1, {}, [], False, False))
			self.assertTrue(tempPbConfig.globalDb.updateSearchOrder(False))
			self.assertEqual([e["count"] for e in \
				tempPbConfig.globalDb.getSearchList(False, False)], [2])

			self.assertEqual(tempPbConfig.globalDb.countSearches(), 5)
			for idS in [e["idS"]
					for e in tempPbConfig.globalDb.getAllSearches()]:
				self.assertTrue(tempPbConfig.globalDb.deleteSearch(idS))
			self.assertEqual(tempPbConfig.globalDb.countSearches(), 0)

class TestProfilesDB(unittest.TestCase):
	"""Test GlobalDB"""
	@patch('sys.stdout', new_callable=StringIO)
	def assert_in_stdout(self, function, expected_output, mock_stdout):
		"""Catch and if test stdout of the function contains a string"""
		pBErrorManager.tempHandler(sys.stdout, format='%(message)s')
		function()
		pBErrorManager.rmTempHandler()
		self.assertIn(expected_output, mock_stdout.getvalue())

	def test_GlobalDB(self):
		"""Test database for profiles"""
		if os.path.exists(tempProfName):
			os.remove(tempProfName)
		self.globalDb = GlobalDB(tempProfName,
			pbConfig.logger, pbConfig.dataPath, info=False)
		self.assertEqual(self.globalDb.countProfiles(), 1)
		self.assertEqual(self.globalDb.getDefaultProfile(), "default")

		with self.assertRaises(SystemExit):
			self.assertFalse(self.globalDb.createProfile())
		with self.assertRaises(SystemExit):
			self.assertFalse(self.globalDb.createProfile("default1"))
		with self.assertRaises(SystemExit):
			self.assertFalse(self.globalDb.createProfile(
				databasefile="database1.db"))

		self.assertEqual(self.globalDb.countProfiles(), 1)
		self.assertTrue(self.globalDb.createProfile("temp1",
			"d", "somefile.db", "old"))
		self.assertEqual(self.globalDb.countProfiles(), 2)
		self.assertEqual(self.globalDb.getDefaultProfile(), "default")
		self.assertEqual(self.globalDb.getProfileOrder(), ["default", "temp1"])

		output = {"name": "temp1",
			"description": "d",
			"databasefile": "somefile.db",
			"oldCfg": "old",
			"isDefault": 0,
			"ord": 100}
		self.assertEqual(dict(self.globalDb.getProfile("temp1")), output)
		self.assertTrue(self.globalDb.updateProfileField("temp1",
			"description", "desc"))
		output["description"] = "desc"
		self.assertEqual(dict(self.globalDb.getProfile("temp1")), output)
		self.assertTrue(self.globalDb.updateProfileField("somefile.db",
			"name", "temp", "databasefile"))
		output["name"] = "temp"

		self.assertEqual(dict(self.globalDb.getProfile("temp")), output)
		self.assertEqual(dict(self.globalDb.getProfile(
			filename="somefile.db")), output)
		self.assertEqual(dict(self.globalDb.getProfile()), {})
		self.assertEqual(dict(self.globalDb.getProfile("temp", "somefile.db")),
			{})

		self.assertFalse(self.globalDb.updateProfileField("tmp",
			"name", "temp", "name"))
		self.assertFalse(self.globalDb.updateProfileField("tmp",
			"databasefile", "temp", "isDefault"))
		self.assertFalse(self.globalDb.updateProfileField("tmp",
			"name", "temp", "isDefault"))
		self.assertFalse(self.globalDb.updateProfileField("tmp",
			"name1", "temp"))

		self.assertTrue(self.globalDb.updateProfileField(1,
			"description", "newsomething", "isDefault"))
		output["name"] = "temp"
		self.assertEqual(dict(self.globalDb.getProfile("temp")), output)

		output = [dict(e) for e in self.globalDb.getProfiles()]
		self.assertEqual(output, [{"name": u"default",
			"description": u"newsomething",
			"databasefile": str(os.path.join(pbConfig.dataPath,
				config_defaults["mainDatabaseName"].replace("PBDATA", ""))),
			"oldCfg": u"",
			"isDefault": 1,
			"ord": 100},
			{"name": u"temp",
			"description": u"desc",
			"databasefile": u"somefile.db",
			"oldCfg": u"old",
			"isDefault": 0,
			"ord": 100}])

		self.assertEqual(self.globalDb.getProfileOrder(),
			["default", "temp"])
		self.assertTrue(self.globalDb.updateProfileField(
			"somefile.db", "name", "abc", "databasefile"))
		self.assertEqual(self.globalDb.getProfileOrder(), ["abc", "default"])
		self.assertTrue(self.globalDb.setProfileOrder(["default", "abc"]))
		self.assertEqual(self.globalDb.getProfileOrder(), ["default", "abc"])
		self.assertFalse(self.globalDb.setProfileOrder())
		self.assert_in_stdout(self.globalDb.setProfileOrder, "No order given!")
		self.assertFalse(self.globalDb.setProfileOrder(["default", "temp"]))
		self.assert_in_stdout(lambda: self.globalDb.setProfileOrder(
			["default", "temp"]),
			"List of profile names does not match existing profiles!")
		with patch("physbiblio.databaseCore.PhysBiblioDBCore.connExec",
				side_effect=[True, False, True, False],
				autospec=True) as _mock:
			self.assertFalse(self.globalDb.setProfileOrder(["abc", "default"]))
			self.assert_in_stdout(lambda: self.globalDb.setProfileOrder(
				["abc", "default"]),
				"Something went wrong when setting new profile order. "
				+ "Undoing...")

		self.assertEqual(self.globalDb.getDefaultProfile(), "default")
		self.assertTrue(self.globalDb.setDefaultProfile("abc"))
		self.assertEqual(self.globalDb.getDefaultProfile(), "abc")
		self.assertFalse(self.globalDb.setDefaultProfile())
		self.assert_in_stdout(self.globalDb.setDefaultProfile,
			"No name given!")
		self.assertFalse(self.globalDb.setDefaultProfile("temp"))
		self.assert_in_stdout(lambda: self.globalDb.setDefaultProfile("temp"),
			"No profiles with the given name!")
		with patch("physbiblio.databaseCore.PhysBiblioDBCore.connExec",
				side_effect=[True, False, True, False],
				autospec=True) as _mock:
			self.assertFalse(self.globalDb.setDefaultProfile("abc"))
			self.assert_in_stdout(
				lambda: self.globalDb.setDefaultProfile("abc"),
				"Something went wrong when setting new default profile. "
				+ "Undoing...")
		self.assertEqual(self.globalDb.getDefaultProfile(), "abc")

		self.assertFalse(self.globalDb.deleteProfile(""))
		self.assertTrue(self.globalDb.deleteProfile("temp1"))
		self.assertEqual(self.globalDb.countProfiles(), 2)
		self.assertTrue(self.globalDb.deleteProfile("abc"))
		self.assertEqual(self.globalDb.countProfiles(), 1)
		self.assertEqual(self.globalDb.getDefaultProfile(), "default")

@unittest.skipIf(skipTestsSettings.db, "Database tests")
class TestConfigDB(DBTestCase):
	"""Test ConfigurationDB"""
	def test_configDB(self):
		"""Test count, insert and update, delete and get methods"""
		self.assertEqual(self.pBDB.config.count(), 0)
		self.assertTrue(self.pBDB.config.insert("test", "somevalue"))
		self.assertEqual(self.pBDB.config.count(), 1)
		self.assertTrue(self.pBDB.config.insert("test1", "somevalue"))
		self.assertEqual(self.pBDB.config.count(), 2)
		self.assertTrue(self.pBDB.config.insert("test", "somevalue1"))
		self.assertEqual(self.pBDB.config.count(), 2)
		self.assertTrue(self.pBDB.config.update("test1", "somevalueA"))
		self.assertEqual(self.pBDB.config.count(), 2)
		self.assertTrue(self.pBDB.config.update("test2", "somevalue"))
		self.assertEqual(self.pBDB.config.count(), 3)
		for n, v in [
				["test", "somevalue1"],
				["test1", "somevalueA"],
				["test2", "somevalue"]]:
			self.assertEqual(self.pBDB.config.getByName(n)[0]["value"], v)
		self.assertEqual({e["name"]: e["value"] for e in \
			self.pBDB.config.getAll()},
			{"test": "somevalue1",
			"test1": "somevalueA",
			"test2": "somevalue"})
		self.assertTrue(self.pBDB.config.delete("test3"))
		self.assertEqual(self.pBDB.config.count(), 3)
		self.assertTrue(self.pBDB.config.delete("test2"))
		self.assertEqual(self.pBDB.config.count(), 2)
		if os.path.exists(tempDBName):
			os.remove(tempDBName)

def tearDownModule():
	if os.path.exists(tempCfgName):
		os.remove(tempCfgName)
	if os.path.exists(tempCfgName1):
		os.remove(tempCfgName1)
	if os.path.exists(tempProfName):
		os.remove(tempProfName)
	if os.path.exists(tempOldCfgName):
		os.remove(tempOldCfgName)

if __name__=='__main__':
	unittest.main()



"""Module with the classes and functions that manage
the experiments windows and panels.

This file is part of the physbiblio package.
"""
import traceback
from PySide2.QtCore import Qt, QTimer
from PySide2.QtGui import QCursor
from PySide2.QtWidgets import QAction, QLineEdit, QPushButton, QToolTip

try:
	from physbiblio.config import pbConfig
	from physbiblio.database import pBDB
	from physbiblio.view import pBView
	from physbiblio.errors import pBLogger
	from physbiblio.gui.errorManager import pBGUILogger
	from physbiblio.gui.basicDialogs import askYesNo
	from physbiblio.gui.commonClasses import \
		EditObjectWindow, PBLabel, PBMenu, PBTableModel, \
		ObjListWindow, pBGuiView
	from physbiblio.gui.catWindows import CatsTreeWindow
	import physbiblio.gui.resourcesPyside2
except ImportError:
	print("Could not find physbiblio and its modules!")
	print(traceback.format_exc())


def editExperiment(parentObject,
		mainWinObject,
		editIdExp=None):
	"""Open a dialog (`EditExperimentDialog`) to edit an experiment
	and process the output.

	Parameters:
		parentObject: the parent widget
		mainWinObject: the object which has
			the `statusBarMessage` and `setWindowTitle` methods
		editIdCat: the id of the experiment to be edited,
			or `None` to create a new one
	"""
	if editIdExp is not None:
		edit = pBDB.exps.getDictByID(editIdExp)
	else:
		edit = None
	newExpWin = EditExperimentDialog(parentObject, experiment=edit)
	newExpWin.exec_()
	if newExpWin.result:
		data = {}
		for k, v in newExpWin.textValues.items():
			s = "%s"%v.text()
			data[k] = s
		if data["name"].strip() != "":
			if "idExp" in data.keys():
				pBLogger.info("Updating experiment %s..."%data["idExp"])
				pBDB.exps.update(data, data["idExp"])
			else:
				pBDB.exps.insert(data)
			message = "Experiment saved"
			mainWinObject.setWindowTitle("PhysBiblio*")
			try:
				parentObject.recreateTable()
			except AttributeError:
				pBLogger.debug("parentObject has no attribute 'recreateTable'",
					exc_info=True)
		else:
			message = "ERROR: empty experiment name"
	else:
		message = "No modifications to experiments"
	try:
		mainWinObject.statusBarMessage(message)
	except AttributeError:
		pBLogger.debug("mainWinObject has no attribute 'statusBarMessage'",
			exc_info=True)


def deleteExperiment(parentObject, mainWinObject, idExp, name):
	"""Ask confirmation and eventually delete the selected experiment

	Parameters:
		parentObject: the parent widget
		mainWinObject: the object which has
			the `statusBarMessage` and `setWindowTitle` methods
		idExp: the id of the experiment to be deleted
		name: the name of the experiment to be deleted
	"""
	if askYesNo("Do you really want to delete this experiment "
			+ "(ID = '%s', name = '%s')?"%(idExp, name)):
		pBDB.exps.delete(int(idExp))
		mainWinObject.setWindowTitle("PhysBiblio*")
		message = "Experiment deleted"
		try:
			parentObject.recreateTable()
		except AttributeError:
			pBLogger.debug("parentObject has no attribute 'recreateTable'",
				exc_info=True)
	else:
		message = "Nothing changed"
	try:
		mainWinObject.statusBarMessage(message)
	except AttributeError:
		pBLogger.debug("mainWinObject has no attribute 'statusBarMessage'",
			exc_info=True)


class ExpTableModel(PBTableModel):
	"""Model for the experiment list"""

	typeClass = "Exps"

	def __init__(self,
			parent,
			exp_list,
			header,
			askExps=False,
			previous=[],
			*args):
		"""Extension of `PBTableModel`
		Initialize the model, save the data and the initial selection

		Parameters:
			parent: the parent widget
			exp_list: the list of experiments records from the database
			header: the names of the column fields
			askExps (default False): if True, enable checkboxes for
				selection of experiments
			previous: the list of previously selected experiments
				(default: an empty list)
		"""
		self.dataList = exp_list
		super(ExpTableModel, self).__init__(
			parent, header, askExps, previous, *args)
		self.prepareSelected()

	def getIdentifier(self, element):
		"""Get the string that identifies the given element
		in the database

		Parameter:
			element: the database record
		"""
		return element["idExp"]

	def data(self, index, role):
		"""Return the cell data for the given index and role

		Parameters:
			index: the `QModelIndex` for which the data are required
			role: the desired Qt role for the data

		Output:
			None if the index or the role are not valid,
			the cell content or properties otherwise
		"""
		if not index.isValid():
			return None
		row = index.row()
		column = index.column()
		try:
			value = self.dataList[row][self.header[column]]
		except IndexError:
			return None

		if role == Qt.CheckStateRole and self.ask and column == 0:
			if self.selectedElements[self.dataList[row][self.header[0]]
					] == False:
				return Qt.Unchecked
			else:
				return Qt.Checked
		if role == Qt.EditRole:
			return value
		if role == Qt.DisplayRole:
			return value
		return None

	def setData(self, index, value, role):
		"""Set the cell data for the given index and role

		Parameters:
			index: the `QModelIndex` for which the data are required
			value: the new data value
			role: the desired Qt role for the data

		Output:
			True if correctly completed,
			False if the `index` is not valid
		"""
		if not index.isValid():
			return False
		if role == Qt.CheckStateRole and index.column() == 0:
			if value == Qt.Checked:
				self.selectedElements[self.dataList[index.row()]["idExp"]] \
					= True
			else:
				self.selectedElements[self.dataList[index.row()]["idExp"]] \
					= False

		self.dataChanged.emit(index, index)
		return True


class ExpsListWindow(ObjListWindow):
	"""create a window for printing the list of experiments"""

	colcnt = len(pBDB.tableCols["experiments"])
	colContents = [pBDB.tableCols["experiments"][j]
		for j in range(len(pBDB.tableCols["experiments"]))]

	def __init__(self,
			parent=None,
			askExps=False,
			askForBib=None,
			askForCat=None,
			previous=[]):
		"""Constructor, extends `ObjListWindow.__init__`
		with more settings and parameters

		Parameters:
			parent: the parent widget
			askExps: if True, checkboxes will be used
			askForBib: the key identifying the bibtex entry for which
				the experiments are asked
			askForCat: the ID identifying the category for which
				the experiments are asked
			previous: the list (default empty) of initially selected
				experiments
		"""
		self.askExps = askExps
		self.askForBib = askForBib
		self.askForCat = askForCat
		self.previous = previous
		self.marked = []
		self.result	= False
		self.exps = []
		self.newExpButton = None
		self.acceptButton = None
		self.cancelButton = None
		self.menu = None
		self.timer = None

		ObjListWindow.__init__(self, parent)
		self.setWindowTitle('List of experiments')

		self.createTable()

	def populateAskExp(self):
		"""If selection of experiments is allowed, add some information
		on the bibtex/category for which the experiments are requested
		and a simple message, then create few required empty lists
		"""
		if self.askExps:
			if self.askForBib is not None:
				try:
					bibitem = pBDB.bibs.getByBibkey(
						self.askForBib, saveQuery=False)[0]
				except IndexError:
					pBGUILogger.warning("The entry '%s' "%self.askForBib
						+ "is not in the database!", exc_info=True)
					return
				try:
					if bibitem["inspire"] != "" and \
							bibitem["inspire"] is not None:
						link = "<a href='%s'>%s</a>"%(
							pBView.getLink(self.askForBib, "inspire"),
							self.askForBib)
					elif bibitem["arxiv"] != "" and \
							bibitem["arxiv"] is not None:
						link = "<a href='%s'>%s</a>"%(
							pBView.getLink(self.askForBib, "arxiv"),
							self.askForBib)
					elif bibitem["doi"] != "" and \
							bibitem["doi"] is not None:
						link = "<a href='%s'>%s</a>"%(
							pBView.getLink(self.askForBib, "doi"),
							self.askForBib)
					else:
						link = self.askForBib
					bibtext = PBLabel(
						"Mark experiments for the following entry:<br>"
						+ "<b>key</b>:<br>%s<br>"%link
						+ "<b>author(s)</b>:<br>%s<br>"%bibitem["author"]
						+ "<b>title</b>:<br>%s<br>"%bibitem["title"])
				except KeyError:
					bibtext = PBLabel(
						"Mark experiments for the following entry:<br>"
						+ "<b>key</b>:<br>%s<br>"%(self.askForBib))
				self.currLayout.addWidget(bibtext)
			elif self.askForCat is not None:
				raise NotImplementedError("This feature is not implemented")
			else:
				self.currLayout.addWidget(
					PBLabel("Select the desired experiments:"))
			self.marked = []
			self.parent().selectedExps = []
		return True

	def onCancel(self):
		"""Reject the dialog content and close the window"""
		self.result	= False
		self.close()

	def onOk(self):
		"""Accept the dialog content
		(update the list of selected experiments)
		and close the window.
		"""
		self.parent().selectedExps = [
			idE for idE in self.tableModel.selectedElements.keys() \
			if self.tableModel.selectedElements[idE] == True]
		self.result	= "Ok"
		self.close()

	def onNewExp(self):
		"""Action to perform when the creation
		of a new experiment is requested
		"""
		editExperiment(self, self.parent())

	def keyPressEvent(self, e):
		"""Manage the key press events.
		Do nothing unless `Esc` is pressed:
		in this case close the dialog
		"""
		if e.key() == Qt.Key_Escape:
			self.close()

	def createTable(self):
		"""Create the dialog content, connect the model to the view
		and eventually add the buttons at the end
		"""
		self.populateAskExp()

		self.exps = pBDB.exps.getAll()

		self.tableModel = ExpTableModel(self,
			self.exps,
			pBDB.tableCols["experiments"],
			askExps=self.askExps,
			previous=self.previous)
		self.addFilterInput("Filter experiment")
		self.setProxyStuff(1, Qt.AscendingOrder)

		self.finalizeTable()

		self.newExpButton = QPushButton('Add new experiment', self)
		self.newExpButton.clicked.connect(self.onNewExp)
		self.currLayout.addWidget(self.newExpButton)

		if self.askExps:
			self.acceptButton = QPushButton('OK', self)
			self.acceptButton.clicked.connect(self.onOk)
			self.currLayout.addWidget(self.acceptButton)

			# cancel button
			self.cancelButton = QPushButton('Cancel', self)
			self.cancelButton.clicked.connect(self.onCancel)
			self.cancelButton.setAutoDefault(True)
			self.currLayout.addWidget(self.cancelButton)

	def triggeredContextMenuEvent(self, row, col, event):
		"""Process event when mouse right-clicks an item.
		Opens a menu with a number of actions

		Parameter:
			row: the row number
			col: a the column number
			event: a `QEvent` instance, used to obtain the mouse
				position where to open the menu
		"""
		index = self.proxyModel.index(row, col)
		if not index.isValid():
			return
		try:
			idExp = str(self.proxyModel.sibling(row, 0, index).data())
			expName = str(self.proxyModel.sibling(row, 1, index).data())
		except AttributeError:
			return

		menu = PBMenu()
		self.menu = menu
		titAction = QAction("--Experiment: %s--"%expName)
		titAction.setDisabled(True)
		bibAction = QAction("Open list of corresponding entries")
		modAction = QAction("Modify")
		delAction = QAction("Delete")
		catAction = menu.addAction("Categories")
		menu.possibleActions = [
			titAction, None,
			bibAction, None,
			modAction, delAction, None,
			catAction
			]
		menu.fillMenu()

		action = menu.exec_(event.globalPos())

		if action == bibAction:
			self.parent().reloadMainContent(pBDB.bibs.getByExp(idExp))
		elif action == modAction:
			editExperiment(self, self.parent(), idExp)
		elif action == delAction:
			deleteExperiment(self, self.parent(), idExp, expName)
		elif action == catAction:
			previous = [a["idCat"] for a in pBDB.cats.getByExp(idExp)]
			selectCats = CatsTreeWindow(
				parent=self.parent(),
				askCats=True,
				askForExp=idExp,
				expButton=False,
				previous=previous)
			selectCats.exec_()
			if selectCats.result == "Ok":
				cats = self.parent().selectedCats
				for p in previous:
					if p not in cats:
						pBDB.catExp.delete(p, idExp)
				for c in cats:
					if c not in previous:
						pBDB.catExp.insert(c, idExp)
				self.parent().statusBarMessage(
					"Categories for '%s' successfully inserted"%expName)
		return True

	def handleItemEntered(self, index):
		"""Process event when mouse enters an item and
		create a `QTooltip` which describes the category, with a timer

		Parameter:
			index: a `QModelIndex` instance
		"""
		if index.isValid():
			row = index.row()
		else:
			return
		idExp = str(self.proxyModel.sibling(row, 0, index).data())
		try:
			self.timer.stop()
			QToolTip.showText(QCursor.pos(), "", self.tableview.viewport())
		except AttributeError:
			pass
		try:
			expData = pBDB.exps.getByID(idExp)[0]
		except IndexError:
			pBGUILogger.exception("Failed in finding experiment")
			return
		self.timer = QTimer(self)
		self.timer.setSingleShot(True)
		self.timer.timeout.connect(lambda: QToolTip.showText(
			QCursor.pos(),
			"{idE}: {exp}\nCorresponding entries: {en}\n".format(
				idE=idExp,
				exp=expData["name"],
				en=pBDB.bibExp.countByExp(idExp))
			+ "Associated categories: {ca}".format(
				ca=pBDB.catExp.countByExp(idExp)),
			self.tableview.viewport(),
			self.tableview.visualRect(index),
			3000))
		self.timer.start(500)

	def cellClick(self, index):
		"""Process event when mouse clicks an item.
		Currently not used

		Parameter:
			index: a `QModelIndex` instance
		"""
		if index.isValid():
			row = index.row()
		else:
			return
		idExp = str(self.proxyModel.sibling(row, 0, index).data())
		return True

	def cellDoubleClick(self, index):
		"""Process event when mouse double clicks an item.
		Opens a link if some columns

		Parameter:
			index: a `QModelIndex` instance
		"""
		if index.isValid():
			row = index.row()
			col = index.column()
		else:
			return
		idExp = str(self.proxyModel.sibling(row, 0, index).data())
		if self.colContents[col] == "inspire" or \
				self.colContents[col] == "homepage":
			link = self.proxyModel.sibling(row, col, index).data()
			if link == "" or link is None:
				return
			if self.colContents[col] == "inspire":
				link = pbConfig.inspireRecord + link
			try:
				pBLogger.debug("Opening '%s'..."%link)
				pBGuiView.openLink(link, "link")
			except Exception:
				pBLogger.warning("Opening link '%s' failed!"%link,
					exc_info=True)
			return True
		return None


class EditExperimentDialog(EditObjectWindow):
	"""create a window for editing or creating an experiment"""

	def __init__(self, parent=None, experiment=None):
		"""Extend `EditObjectWindow.__init__` to define self.data
		and call createForm

		Parameters:
			parent: the parent widget
			experiment: the database record or dictionary which
				contains the experiment information
		"""
		super(EditExperimentDialog, self).__init__(parent)
		if experiment is None:
			self.data = {}
			for k in pBDB.tableCols["experiments"]:
				self.data[k] = ""
		else:
			self.data = experiment
		self.acceptButton = None
		self.cancelButton = None
		self.createForm()

	def createForm(self):
		"""Create the form labels, fields, buttons"""
		self.setWindowTitle('Edit experiment')

		i = 0
		for k in pBDB.tableCols["experiments"]:
			val = self.data[k]
			if k != "idExp" or (k == "idExp" and self.data[k] != ""):
				i += 1
				self.currGrid.addWidget(PBLabel(k), i*2 - 1, 0)
				self.currGrid.addWidget(
					PBLabel("(%s)"%pBDB.descriptions["experiments"][k]),
					i*2 - 1, 1)
				self.textValues[k] = QLineEdit(str(val))
				if k == "idExp":
					self.textValues[k].setEnabled(False)
				self.currGrid.addWidget(self.textValues[k], i*2, 0, 1, 2)

		# OK button
		self.acceptButton = QPushButton('OK', self)
		self.acceptButton.clicked.connect(self.onOk)
		self.currGrid.addWidget(self.acceptButton, i*2 + 1, 0)

		# cancel button
		self.cancelButton = QPushButton('Cancel', self)
		self.cancelButton.clicked.connect(self.onCancel)
		self.cancelButton.setAutoDefault(True)
		self.currGrid.addWidget(self.cancelButton, i*2 + 1, 1)

		self.setGeometry(100, 100, 400, 50*i)
		self.centerWindow()



#!/usr/bin/env python
"""Test file for the physbiblio.errors module.

This file is part of the physbiblio package.
"""
import sys
import traceback

if sys.version_info[0] < 3:
	import unittest2 as unittest
	from mock import patch
	from StringIO import StringIO
else:
	import unittest
	from unittest.mock import patch
	from io import StringIO

try:
	from physbiblio.setuptests import *
	from physbiblio.errors import PBErrorManagerClass
	from physbiblio.config import pbConfig
except ImportError:
	print("Could not find physbiblio and its modules!")
	raise
except Exception:
	print(traceback.format_exc())


class TestErrors(unittest.TestCase):
	"""Test PBErrorManagerClass stuff"""
	@classmethod
	def setUpClass(self):
		"""settings"""
		with patch.dict(pbConfig.params,
				{"loggingLevel": 3, "logFileName": logFileName},
				clear=False):
			self.pBErrorManager = PBErrorManagerClass("testlogger")

	def reset(self, m):
		"""Clear a StringIO object"""
		m.seek(0)
		m.truncate(0)

	def resetLogFile(self):
		"""Reset the logfile content"""
		open(logFileName, 'w').close()

	def readLogFile(self):
		"""Read the logfile content"""
		with open(logFileName) as logFile:
			log_new = logFile.read()
		return log_new

	def test_critical(self):
		"""Test pBLogger.critical with and without exception"""
		self.resetLogFile()
		try:
			raise Exception("Fake error")
		except Exception as e:
			self.pBErrorManager.logger.critical(str(e))
		log_new = self.readLogFile()
		self.assertIn(datetime.datetime.now().strftime("%Y-%m-%d %H:%M"),
			log_new)
		self.assertIn("CRITICAL : [test_errors.test_critical] Fake error",
			log_new)
		self.assertNotIn("Traceback (most recent call last):", log_new)
		try:
			raise Exception("Fake error")
		except Exception as e:
			self.pBErrorManager.logger.critical(str(e), exc_info=True)
		log_new = self.readLogFile()
		self.assertIn("Traceback (most recent call last):", log_new)

	def test_exception(self):
		"""Test pBLogger.exception"""
		self.resetLogFile()
		try:
			raise Exception("Fake error")
		except Exception as e:
			self.pBErrorManager.logger.exception(str(e))
		try:
			raise Exception("New fake error")
		except Exception as e:
			self.pBErrorManager.logger.error(str(e))
		log_new = self.readLogFile()
		self.assertIn(datetime.datetime.now().strftime("%Y-%m-%d %H:%M"),
			log_new)
		self.assertIn("ERROR : [test_errors.test_exception] Fake error",
			log_new)
		self.assertIn("ERROR : [test_errors.test_exception] New fake error",
			log_new)
		self.assertIn("Traceback (most recent call last):", log_new)

	def test_warning(self):
		"""Test pBLogger.warning"""
		self.resetLogFile()
		try:
			raise Exception("Fake warning")
		except Exception as e:
			self.pBErrorManager.logger.warning(str(e))
		log_new = self.readLogFile()
		self.assertIn(datetime.datetime.now().strftime("%Y-%m-%d %H:%M"),
			log_new)
		self.assertIn("WARNING : [test_errors.test_warning] Fake warning",
			log_new)

	def test_info(self):
		"""Test pBLogger.info"""
		self.resetLogFile()
		self.pBErrorManager.logger.info("Some info")
		log_new = self.readLogFile()
		self.assertIn(datetime.datetime.now().strftime("%Y-%m-%d %H:%M"),
			log_new)
		self.assertIn("INFO : [test_errors.test_info] Some info", log_new)

	def test_tempHandler(self):
		"""Test stuff as the temporary handler"""
		self.assertEqual(self.pBErrorManager.tempsh, [])
		self.assertEqual(len(self.pBErrorManager.logger.handlers), 2)
		with patch('sys.stdout', new_callable=StringIO) as _mock:
			self.assertTrue(self.pBErrorManager.tempHandler(sys.stdout,
				format='%(message)s'))
			self.assertEqual(len(self.pBErrorManager.logger.handlers), 3)
			self.assertEqual(len(self.pBErrorManager.tempsh), 1)
			self.reset(_mock)
			self.pBErrorManager.logger.critical("Fake critical")
			self.assertEqual(_mock.getvalue(), "Fake critical\n")
			self.pBErrorManager.rmTempHandler()
			self.assertEqual(self.pBErrorManager.tempsh, [])
			self.assertEqual(len(self.pBErrorManager.logger.handlers), 2)
			self.assertEqual(len(self.pBErrorManager.tempsh), 0)
			self.reset(_mock)
			self.assertEqual(_mock.getvalue(), "")
			self.pBErrorManager.logger.critical("Fake critical")
			self.assertEqual(_mock.getvalue(), "")
			self.assertTrue(self.pBErrorManager.tempHandler(sys.stdout,
				format='%(message)s'))
			self.assertEqual(len(self.pBErrorManager.logger.handlers), 3)
			self.assertTrue(self.pBErrorManager.tempHandler(sys.stdout,
				format='%(message)s'))
			self.assertEqual(len(self.pBErrorManager.logger.handlers), 4)
			self.assertTrue(self.pBErrorManager.tempHandler(sys.stdout,
				format='%(message)s'))
			self.assertEqual(len(self.pBErrorManager.logger.handlers), 5)
			self.assertEqual(len(self.pBErrorManager.tempsh), 3)
			self.pBErrorManager.rmTempHandler()
			self.assertEqual(len(self.pBErrorManager.logger.handlers), 4)
			self.assertEqual(len(self.pBErrorManager.tempsh), 2)
			self.pBErrorManager.rmAllTempHandlers()
			self.assertEqual(len(self.pBErrorManager.logger.handlers), 2)
			self.assertEqual(len(self.pBErrorManager.tempsh), 0)


if __name__=='__main__':
	unittest.main()



#!/usr/bin/env python
"""Test file for the physbiblio.view module.

This file is part of the physbiblio package.
"""
import sys
import traceback

if sys.version_info[0] < 3:
	import unittest2 as unittest
	from mock import call, patch
else:
	import unittest
	from unittest.mock import call, patch

try:
	from physbiblio.setuptests import *
	from physbiblio.config import pbConfig
	from physbiblio.database import pBDB
	from physbiblio.view import *
except ImportError:
	print("Could not find physbiblio and its modules!")
	raise
except Exception:
	print(traceback.format_exc())


@unittest.skipIf(skipTestsSettings.long, "Long tests")
class TestViewMethods(unittest.TestCase):
	"""Tests for methods in physbiblio.view"""

	def test_init(self):
		"""test instance properties"""
		tmp = ViewEntry()
		self.assertIsInstance(tmp, ViewEntry)
		self.assertEqual(tmp.webApp, pbConfig.params["webApplication"])
		self.assertEqual(tmp.inspireRecord, pbConfig.inspireRecord)
		self.assertEqual(tmp.inspireSearch,
			pbConfig.inspireSearchBase + "?p=find+")
		with patch.dict(pbConfig.params,
				{"webApplication": "someapp"},
				clear=False):
			tmp = ViewEntry()
			self.assertEqual(tmp.webApp, "someapp")

	def test_getLink(self):
		"""Test getLink function with different inputs"""
		with patch('physbiblio.database.Entries.getField',
				autospec=True,
				side_effect=[
				'1507.08204', '', '', '1507.08204', #test "arxiv"
				'', '10.1088/0954-3899/43/3/033001',
					'', '10.1088/0954-3899/43/3/033001', #test "doi"
				'', '', '1385583', #test "inspire", inspire ID present
				'1507.08204', '', '', #test "inspire",
					# inspire ID not present, arxiv present
				'', '', False, #test "inspire",
					# inspire ID not present, arxiv not present
				'', '', '', #test "arxiv", no info
				'', '', '', #test "doi", no info
				]) as _mock:
			self.assertEqual(pBView.getLink("a", "arxiv"),
				"%s/abs/1507.08204"%pbConfig.arxivUrl)
			self.assertEqual(pBView.getLink("a", "doi"),
				"%s10.1088/0954-3899/43/3/033001"%pbConfig.doiUrl)
			self.assertEqual(pBView.getLink("a", "inspire"),
				"https://inspirehep.net/record/1385583")
			self.assertEqual(pBView.getLink("a", "inspire"),
				"https://inspirehep.net/search?p=find+1507.08204")
			self.assertEqual(pBView.getLink("a", "inspire"),
				"https://inspirehep.net/search?p=find+a")
			self.assertFalse(pBView.getLink("a", "arxiv"))
			self.assertFalse(pBView.getLink("a", "doi"))

	def test_openLink(self):
		"""Test openLink"""
		origwebapp = pBView.webApp
		with patch("physbiblio.view.ViewEntry.getLink",
				autospec=True, return_value="test_link") as _gl,\
				patch("logging.Logger.info") as _i,\
				patch("logging.Logger.warning") as _w,\
				patch("subprocess.Popen", autospec=True) as _po:
			pBView.openLink(["a", "b"], arg="file")
			_gl.assert_has_calls([
				call(pBView, 'a', arg='file', fileArg=None),
				call(pBView, 'b', arg='file', fileArg=None)])
			_po.assert_not_called()
			_gl.reset_mock()
			pBView.webApp = "test"
			pBView.openLink("a")
			_gl.assert_called_once_with(pBView,
				'a', arg='arxiv', fileArg=None)
			_po.assert_called_once_with([pBView.webApp, "test_link"],
				stdout=subprocess.PIPE,
				stderr=subprocess.STDOUT)
			_i.assert_called_once_with("Opening 'test_link'...")
			_w.assert_not_called()
			_i.reset_mock()
			pBView.webApp = ""
			pBView.openLink("a")
			_i.assert_not_called()
			_w.assert_not_called()
			_gl.reset_mock()
			_po.reset_mock()
			pBView.webApp = "test"
			pBView.openLink("a", arg="link")
			_po.assert_called_once_with([pBView.webApp, "a"],
				stdout=subprocess.PIPE,
				stderr=subprocess.STDOUT)
			_i.assert_called_once_with("Opening 'a'...")
			_po.side_effect=OSError
			pBView.openLink("a", arg="link")
			_w.assert_called_once_with("Opening link for 'a' failed!")
			_w.reset_mock()
			_gl.reset_mock()
			_gl.return_value = ""
			pBView.openLink("a", arg="doi", fileArg="arg")
			_gl.assert_called_once_with(pBView,
				'a', arg='doi', fileArg="arg")
			_w.assert_called_once_with(
				"Impossible to get the 'doi' link for entry 'a'")
		pBView.webApp = origwebapp


if __name__=='__main__':
	unittest.main()



"""Module with the classes and functions
that manage the some dialog windows.

This file is part of the physbiblio package.
"""
import traceback
import ast
import six
from PySide2.QtCore import Qt, Signal
from PySide2.QtGui import QTextCursor
from PySide2.QtWidgets import \
	QCheckBox, QComboBox, QDesktopWidget, QGridLayout, \
	QLineEdit, QPlainTextEdit, QProgressBar, QPushButton, QTableWidgetItem, \
	QTextEdit, QVBoxLayout

try:
	from physbiblio.config import pbConfig
	from physbiblio.errors import pBLogger
	from physbiblio.database import pBDB
	from physbiblio.webimport.webInterf import physBiblioWeb
	from physbiblio.gui.errorManager import pBGUILogger
	from physbiblio.gui.basicDialogs import \
		askDirName, askFileNames, askSaveFileName, askYesNo, infoMessage
	from physbiblio.gui.commonClasses import \
		PBComboBox, PBDDTableWidget, PBDialog, PBImportedTableModel, \
		PBLabel, PBLabelRight, PBTableView, PBTrueFalseCombo, ObjListWindow
	from physbiblio.gui.catWindows import CatsTreeWindow
	import physbiblio.gui.resourcesPyside2
except ImportError:
	print("Could not find physbiblio and its modules!")
	print(traceback.format_exc())


class ConfigEditColumns(PBDialog):
	"""Extend `PBDialog` to ask the columns
	that must appear in the main table
	"""

	excludeCols = [
		"crossref", "bibtex", "exp_paper", "lecture",
		"phd_thesis", "review", "proceeding", "book", "noUpdate",
		"bibdict", "abstract"]
	moreCols = [
		"title", "author", "journal", "volume", "pages",
		"primaryclass", "booktitle", "reportnumber"]

	def __init__(self, parent=None, previous=None):
		"""Extend `PBDialog.__init__` and create the form structure

		Parameters:
			parent: teh parent widget
			previous: list of columns which must appear
				as selected at the beginning
		"""
		super(ConfigEditColumns, self).__init__(parent)
		self.gridlayout = None
		self.items = []
		self.listAll = None
		self.listSel = None
		self.allItems = None
		self.selItems = None
		self.previousSelected = previous \
			if previous is not None \
			else pbConfig.params["bibtexListColumns"]
		self.selected = self.previousSelected
		self.result	= False
		self.acceptButton = None
		self.cancelButton = None
		self.initUI()

	def onCancel(self):
		"""Reject the output (set self.result to False and close)"""
		self.result	= False
		self.close()

	def onOk(self):
		"""Accept the output and prepare `self.selected`"""
		self.result = True
		self.selected = []
		for row in range(self.listSel.rowCount()):
			self.selected.append(self.listSel.item(row, 0).text())
		self.close()

	def initUI(self):
		"""Initialize the `PBDDTableWidget`s and their content,
		plus the buttons and labels
		"""
		self.gridlayout = QGridLayout()
		self.setLayout(self.gridlayout)

		self.items = []
		self.listAll = PBDDTableWidget(self, "Available columns")
		self.listSel = PBDDTableWidget(self, "Selected columns")
		self.gridlayout.addWidget(
			PBLabel("Drag and drop items to order visible columns"),
			0, 0, 1, 2)
		self.gridlayout.addWidget(self.listAll, 1, 0)
		self.gridlayout.addWidget(self.listSel, 1, 1)

		self.allItems = list(pBDB.descriptions["entries"]) + self.moreCols
		self.selItems = self.previousSelected
		isel = 0
		iall = 0
		for col in self.selItems \
				+ [i for i in self.allItems if i not in self.selItems]:
			if col in self.excludeCols:
				continue
			item = QTableWidgetItem(col)
			self.items.append(item)
			if col in self.selItems:
				self.listSel.insertRow(isel)
				self.listSel.setItem(isel, 0, item)
				isel += 1
			else:
				self.listAll.insertRow(iall)
				self.listAll.setItem(iall, 0, item)
				iall += 1

		self.acceptButton = QPushButton('OK', self)
		self.acceptButton.clicked.connect(self.onOk)
		self.gridlayout.addWidget(self.acceptButton, 2, 0)

		self.cancelButton = QPushButton('Cancel', self)
		self.cancelButton.clicked.connect(self.onCancel)
		self.cancelButton.setAutoDefault(True)
		self.gridlayout.addWidget(self.cancelButton, 2, 1)


class ConfigWindow(PBDialog):
	"""Create a window for editing the configuration settings"""

	def __init__(self, parent=None):
		"""Simple extension of `PBDialog.__init__`"""
		super(ConfigWindow, self).__init__(parent)
		self.textValues = []
		self.result	= False
		self.selectedCats = None
		self.selectedExps = None
		self.grid = None
		self.acceptButton = None
		self.cancelButton = None
		self.initUI()

	def onCancel(self):
		"""Reject the output (set self.result to False and close)"""
		self.result	= False
		self.close()

	def onOk(self):
		"""Accept the output (set self.result to True and close)"""
		self.result	= True
		self.close()

	def editPDFFolder(self):
		"""Open a dialog to select a new folder name
		for the PDF path, and save the result
		in the `ConfigWindow` interface
		"""
		ix = pbConfig.paramOrder.index("pdfFolder")
		folder = askDirName(
			parent=None,
			dir=self.textValues[ix][1].text(),
			title="Directory for saving PDF files:")
		if folder.strip() != "":
			self.textValues[ix][1].setText(str(folder))

	def editFile(self,
			paramkey="logFileName",
			text="Name for the log file",
			filter="*.log"):
		"""Open a dialog to select a new file name
		for a configuration parameter, and save the result
		in the `ConfigWindow` interface

		Parameters:
			paramkey: the parameter name in the configuration dictionary
			text: description in the file dialog
			filter: filter the folder content in the file dialog
		"""
		if paramkey not in pbConfig.paramOrder:
			pBLogger.warning("Invalid paramkey: '%s'"%paramkey)
			return
		ix = pbConfig.paramOrder.index(paramkey)
		fname = askSaveFileName(
			parent=None,
			title=text,
			dir=self.textValues[ix][1].text(),
			filter=filter)
		if fname.strip() != "":
			self.textValues[ix][1].setText(str(fname))

	def editColumns(self):
		"""Open a dialog to select and/or reorder the list of columns
		to show in the entries list, and save the result
		in the `ConfigWindow` interface
		"""
		ix = pbConfig.paramOrder.index("bibtexListColumns")
		window = ConfigEditColumns(self,
			ast.literal_eval(self.textValues[ix][1].text().strip()))
		window.exec_()
		if window.result:
			columns = window.selected
			self.textValues[ix][1].setText(str(columns))

	def editDefCats(self):
		"""Open a dialog to select a the default categories
		for the imported entries, and save the result
		in the `ConfigWindow` interface
		"""
		ix = pbConfig.paramOrder.index("defaultCategories")
		selectCats = CatsTreeWindow(
			parent=self,
			askCats=True,
			expButton=False,
			previous=ast.literal_eval(self.textValues[ix][1].text().strip()))
		selectCats.exec_()
		if selectCats.result == "Ok":
			self.textValues[ix][1].setText(str(self.selectedCats))

	def initUI(self):
		"""Create and fill the `QGridLayout`"""
		self.setWindowTitle('Configuration')

		grid = QGridLayout()
		self.grid = grid
		grid.setSpacing(1)

		i = 0
		for k in pbConfig.paramOrder:
			i += 1
			val = pbConfig.params[k] \
				if isinstance(pbConfig.params[k], six.string_types) \
				else str(pbConfig.params[k])
			grid.addWidget(
				PBLabel("%s (<i>%s</i>)"%(pbConfig.descriptions[k], k)),
				i-1, 0, 1, 2)
			if k == "bibtexListColumns":
				self.textValues.append([k, QPushButton(val)])
				self.textValues[-1][1].clicked.connect(self.editColumns)
			elif k == "pdfFolder":
				self.textValues.append([k, QPushButton(val)])
				self.textValues[-1][1].clicked.connect(self.editPDFFolder)
			elif k == "loggingLevel":
				try:
					self.textValues.append([k, PBComboBox(self,
						pbConfig.loggingLevels,
						pbConfig.loggingLevels[int(val)])
						])
				except (IndexError, ValueError):
					pBGUILogger.warning("Invalid string for 'loggingLevel' "
						+ "param. Reset to default")
					self.textValues.append([k,
						PBComboBox(self,
							pbConfig.loggingLevels,
							pbConfig.loggingLevels[
								int(pbConfig.defaultsParams["loggingLevel"])]
							)])
			elif k == "logFileName":
				self.textValues.append([k, QPushButton(val)])
				self.textValues[-1][1].clicked.connect(self.editFile)
			elif k == "defaultCategories":
				self.textValues.append([k, QPushButton(val)])
				self.textValues[-1][1].clicked.connect(self.editDefCats)
			elif pbConfig.specialTypes[k] == "boolean":
				self.textValues.append([k, PBTrueFalseCombo(self, val)])
			else:
				self.textValues.append([k, QLineEdit(val)])
			grid.addWidget(self.textValues[i-1][1], i-1, 2, 1, 2)

		# OK button
		self.acceptButton = QPushButton('OK', self)
		self.acceptButton.clicked.connect(self.onOk)
		#width = self.acceptButton.fontMetrics().boundingRect('OK').width() + 7
		#self.acceptButton.setMaximumWidth(width)
		grid.addWidget(self.acceptButton, i, 0)

		# cancel button
		self.cancelButton = QPushButton('Cancel', self)
		self.cancelButton.clicked.connect(self.onCancel)
		self.cancelButton.setAutoDefault(True)
		#width = self.cancelButton.fontMetrics().boundingRect('Cancel').width() + 7
		#self.cancelButton.setMaximumWidth(width)
		grid.addWidget(self.cancelButton, i, 1)

		self.setGeometry(100, 100, 1000, 30*i)
		self.setLayout(grid)


class LogFileContentDialog(PBDialog):
	"""Create a window for showing the logFile content"""

	title = "Log File Content"

	def __init__(self, parent=None):
		"""Instantiate class and create its widgets

		Parameter:
			parent: the parent widget
		"""
		PBDialog.__init__(self, parent)
		self.textEdit = None
		self.closeButton = None
		self.clearButton = None
		self.initUI()

	def clearLog(self):
		"""Ask confirmation, then eventually clear
		the content of the log file
		"""
		if askYesNo("Are you sure you want to clear the log file?"):
			try:
				open(pbConfig.params["logFileName"], "w").close()
			except IOError:
				pBGUILogger.exception("Impossible to clear log file!")
			else:
				infoMessage("Log file cleared.")
				self.close()

	def initUI(self):
		"""Create window layout and buttons,
		read log file content and print it in the `QPlainTextEdit`
		"""
		self.setWindowTitle(self.title)

		grid = QVBoxLayout()
		grid.setSpacing(1)

		grid.addWidget(PBLabel("Reading %s"%pbConfig.params["logFileName"]))
		try:
			with open(pbConfig.params["logFileName"]) as r:
				text = r.read()
		except IOError:
			text = "Impossible to read log file!"
			pBLogger.exception(text)
		self.textEdit = QPlainTextEdit(text)
		self.textEdit.setReadOnly(True)
		grid.addWidget(self.textEdit)

		self.closeButton = QPushButton('Close', self)
		self.closeButton.setAutoDefault(True)
		self.closeButton.clicked.connect(self.close)
		grid.addWidget(self.closeButton)

		self.clearButton = QPushButton('Clear log file', self)
		self.clearButton.clicked.connect(self.clearLog)
		grid.addWidget(self.clearButton)

		self.setGeometry(100, 100, 800, 800)
		self.setLayout(grid)


class PrintText(PBDialog):
	"""Create a window for printing text of command line output"""

	stopped = Signal()

	def __init__(self,
			parent=None,
			title="",
			progressBar=True,
			totStr=None,
			progrStr=None,
			noStopButton=False,
			message=None):
		"""Constructor.
		Set some properties and create the GUI of the dialog

		Parameters:
			parent: the parent widget
			title: the window title. If an empty string,
				"Redirect print" will be used
			progressBar: True (default) if the widget must have a progress bar
			totStr: string to be searched in the printed text in order to
				obtain the number of total iterations to be processed.
				Used to set the progress bar value appropriately.
			progrStr: string to be searched in the printed text in order
				to obtain the iteration number.
				Used to set the progress bar value appropriately.
			noStopButton (default False): True if the widget must have
				a "stop" button to stop the iterations
			message: a text to be inserted as a `PBLabel` in the dialog
		"""
		super(PrintText, self).__init__(parent)
		self._wantToClose = False
		self.grid = None
		self.progressBar = None
		self.textEdit = None
		self.closeButton = None
		self.cancelButton = None
		if title != "":
			self.title = title
		else:
			self.title = "Redirect print"
		self.setProgressBar = progressBar
		self.totString = totStr if totStr is not None else "emptyString"
		self.progressString = progrStr if progrStr is not None \
			else "emptyString"
		self.noStopButton = noStopButton
		self.message = message
		self.initUI()

	def keyPressEvent(self, e):
		"""Intercept press keys and only exit if escape is pressed
		when closing is enabled

		Parameters:
			e: the `PySide2.QtGui.QKeyEvent`
		"""
		if e.key() == Qt.Key_Escape and self._wantToClose:
			self.close()

	def closeEvent(self, event):
		"""Manage the `closeEvent` of the dialog.
		Reject unless `self._wantToClose` is True

		Parameter:
			event: a `QEvent`
		"""
		if self._wantToClose:
			super(PrintText, self).closeEvent(event)
		else:
			event.ignore()

	def initUI(self):
		"""Create the main `QTextEdit`, the buttons and the `QProgressBar`"""
		self.setWindowTitle(self.title)

		grid = QGridLayout()
		self.grid = grid
		grid.setSpacing(1)

		if self.message is not None and self.message.strip() != "":
			grid.addWidget(PBLabel("%s"%self.message))

		self.textEdit = QTextEdit()
		grid.addWidget(self.textEdit)

		if self.setProgressBar:
			self.progressBar = QProgressBar(self)
			grid.addWidget(self.progressBar)

		# cancel button...should learn how to connect it with a thread kill
		if self.noStopButton is not True:
			self.cancelButton = QPushButton('Stop', self)
			self.cancelButton.clicked.connect(self.stopExec)
			self.cancelButton.setAutoDefault(True)
			grid.addWidget(self.cancelButton)
		self.closeButton = QPushButton('Close', self)
		self.closeButton.clicked.connect(lambda: self.reject())
		self.closeButton.setDisabled(True)
		grid.addWidget(self.closeButton)

		self.setGeometry(100, 100, 600, 600)
		self.setLayout(grid)
		self.centerWindow()

	def appendText(self, text):
		"""Add the given text to the end of the `self.textEdit` content.
		If a `self.progressBar` is set, try to obtain
		the maximum and the current value,
		looking for the expected `self.totString` and `self.progressString`

		Parameter:
			text: the string to be appended
		"""
		if self.setProgressBar:
			try:
				if self.totString in text:
					tot = [int(s) for s in text.split() if s.isdigit()][0]
					self.progressBarMax(tot)
				elif self.progressString in text:
					curr = [int(s) for s in text.split() if s.isdigit()][0]
					self.progressBar.setValue(curr)
			except IndexError:
				pBLogger.warning(
					"PrintText.progressBar cannot work with float numbers")
		self.textEdit.moveCursor(QTextCursor.End)
		self.textEdit.insertPlainText(text)

	def progressBarMin(self, minimum):
		"""Set the minimum value for the progress bar

		Parameter:
			minimum (int or float): the value
		"""
		if self.setProgressBar:
			self.progressBar.setMinimum(minimum)

	def progressBarMax(self, maximum):
		"""Set the maximum value for the progress bar

		Parameter:
			maximum (int or float): the value
		"""
		if self.setProgressBar:
			self.progressBar.setMaximum(maximum)

	def stopExec(self):
		"""Stop the iterations through the `stopped` Signal
		and disable the `cancelButton`
		"""
		self.cancelButton.setDisabled(True)
		self.stopped.emit()

	def enableClose(self):
		"""Enable the close button and set `self._wantToClose` to True"""
		self._wantToClose = True
		self.closeButton.setEnabled(True)


class AdvancedImportDialog(PBDialog):
	"""create a window for the advanced import"""

	def __init__(self, parent=None):
		"""Simple extension of `PBDialog.__init__`"""
		super(AdvancedImportDialog, self).__init__(parent)
		self.result	= False
		self.grid = None
		self.searchStr = None
		self.comboMethod = None
		self.acceptButton = None
		self.cancelButton = None
		self.initUI()

	def onCancel(self):
		"""Reject the output (set self.result to False and close)"""
		self.result	= False
		self.close()

	def onOk(self):
		"""Accept the output (set self.result to True and close)"""
		self.result	= True
		self.close()

	def initUI(self):
		"""Create and fill the `QGridLayout`"""
		self.setWindowTitle('Advanced import')

		grid = QGridLayout()
		self.grid = grid
		grid.setSpacing(1)

		##search
		grid.addWidget(PBLabel("Search string: "), 0, 0)
		self.searchStr = QLineEdit("")
		grid.addWidget(self.searchStr, 0, 1)

		grid.addWidget(PBLabel("Select method: "), 1, 0)
		self.comboMethod = PBComboBox(self,
			["INSPIRE-HEP", "arXiv", "DOI", "ISBN"],
			current="INSPIRE-HEP")
		grid.addWidget(self.comboMethod, 1, 1)

		# OK button
		self.acceptButton = QPushButton('OK', self)
		self.acceptButton.clicked.connect(self.onOk)
		grid.addWidget(self.acceptButton, 2, 0)

		# cancel button
		self.cancelButton = QPushButton('Cancel', self)
		self.cancelButton.clicked.connect(self.onCancel)
		self.cancelButton.setAutoDefault(True)
		grid.addWidget(self.cancelButton, 2, 1)

		self.setGeometry(100,100,400, 100)
		self.setLayout(grid)
		self.searchStr.setFocus()
		self.centerWindow()


class AdvancedImportSelect(ObjListWindow):
	"""create a window for the advanced import"""

	def __init__(self, bibs={}, parent=None):
		"""Set some properties and call `initUI`

		Parameters:
			bibs: a dictionary containing the imported bibtex entries.
				Each element should be a dictionary containing at least
				a "bibpars" item, a dictionary with at least the keys
				["ID", "title", "author", "eprint", "doi"],
				and a boolean "exist" item.
			parent: the parent widget
		"""
		self.bibs = bibs
		super(AdvancedImportSelect, self).__init__(parent, gridLayout=True)
		self.checkBoxes = []
		self.result	= False
		self.askCats = None
		self.tableModel = None
		self.askCats = None
		self.acceptButton = None
		self.cancelButton = None
		self.initUI()

	def onCancel(self):
		"""Reject the output (set self.result to False and close)"""
		self.result	= False
		self.close()

	def onOk(self):
		"""Accept the output (set self.result to True and close)"""
		self.selected = self.tableModel.selectedElements
		self.result	= True
		self.close()

	def keyPressEvent(self, e):
		"""Intercept press keys and exit if escape is pressed

		Parameters:
			e: the `PySide2.QtGui.QKeyEvent`
		"""
		if e.key() == Qt.Key_Escape:
			self.result	= False
			self.close()

	def initUI(self):
		"""Create and fill the `QGridLayout`"""
		self.setWindowTitle('Advanced import - results')

		self.currLayout.setSpacing(1)

		self.currLayout.addWidget(PBLabel("This is the list of elements found."
			+ "\nSelect the ones that you want to import:"))

		headers = ["ID", "title", "author", "eprint", "doi"]
		for k in self.bibs.keys():
			try:
				self.bibs[k]['bibpars']["eprint"] = \
					self.bibs[k]['bibpars']["arxiv"]
			except KeyError:
				pass
			try:
				self.bibs[k]['bibpars']["author"] = \
					self.bibs[k]['bibpars']["authors"]
			except KeyError:
				pass
			for f in headers:
				try:
					self.bibs[k]['bibpars'][f] = \
						self.bibs[k]['bibpars'][f].replace("\n", " ")
				except KeyError:
					pass
		self.tableModel = PBImportedTableModel(self, self.bibs, headers)
		self.addFilterInput("Filter entries", gridPos=(1, 0))
		self.setProxyStuff(0, Qt.AscendingOrder)
		self.finalizeTable(gridPos=(2, 0, 1, 2))

		i = 3
		self.askCats = QCheckBox("Ask categories at the end?", self)
		self.askCats.toggle()
		self.currLayout.addWidget(self.askCats, i, 0)

		# OK button
		self.acceptButton = QPushButton('OK', self)
		self.acceptButton.clicked.connect(self.onOk)
		self.currLayout.addWidget(self.acceptButton, i + 1, 0)

		# cancel button
		self.cancelButton = QPushButton('Cancel', self)
		self.cancelButton.clicked.connect(self.onCancel)
		self.cancelButton.setAutoDefault(True)
		self.currLayout.addWidget(self.cancelButton, i + 2, 0)

	def triggeredContextMenuEvent(self, row, col, event):
		"""Does nothing"""
		pass

	def handleItemEntered(self, index):
		"""Does nothing"""
		pass

	def cellClick(self, index):
		"""Does nothing"""
		pass

	def cellDoubleClick(self, index):
		"""Does nothing"""
		pass


class DailyArxivDialog(PBDialog):
	"""create a window for the advanced import"""

	def __init__(self, parent=None):
		"""Simple extension of `PBDialog.__init__`"""
		super(DailyArxivDialog, self).__init__(parent)
		self.result	= False
		self.grid = None
		self.comboSub = None
		self.comboCat = None
		self.acceptButton = None
		self.cancelButton = None
		self.initUI()

	def onCancel(self):
		"""Reject the output (set self.result to False and close)"""
		self.result	= False
		self.close()

	def onOk(self):
		"""Accept the output (set self.result to True and close)"""
		self.result	= True
		self.close()

	def updateCat(self, category):
		"""Replace the current content of the `self.comboSub` combobox
		with the new list of subcategories of the given category

		Parameter:
			category: the name of the category in the
				`physBiblioWeb.webSearch["arxiv"].categories` dictionary
		"""
		self.comboSub.clear()
		self.comboSub.addItems(["--"] \
			+ physBiblioWeb.webSearch["arxiv"].categories[category])

	def initUI(self):
		"""Create and fill the `QGridLayout`"""
		self.setWindowTitle('Browse arxiv daily')

		self.grid = QGridLayout()
		self.grid.setSpacing(1)

		##combo boxes
		self.grid.addWidget(PBLabel("Select category: "), 0, 0)
		self.comboCat = PBComboBox(self,
			[""] + sorted(physBiblioWeb.webSearch["arxiv"].categories.keys()))
		self.comboCat.currentIndexChanged[str].connect(self.updateCat)
		self.grid.addWidget(self.comboCat, 0, 1)

		self.grid.addWidget(PBLabel("Subcategory: "), 1, 0)
		self.comboSub = PBComboBox(self, [""])
		self.grid.addWidget(self.comboSub, 1, 1)

		# OK button
		self.acceptButton = QPushButton('OK', self)
		self.acceptButton.clicked.connect(self.onOk)
		self.grid.addWidget(self.acceptButton, 2, 0)

		# cancel button
		self.cancelButton = QPushButton('Cancel', self)
		self.cancelButton.clicked.connect(self.onCancel)
		self.cancelButton.setAutoDefault(True)
		self.grid.addWidget(self.cancelButton, 2, 1)

		self.setLayout(self.grid)

		self.setGeometry(100, 100, 400, 100)
		self.centerWindow()


class DailyArxivSelect(AdvancedImportSelect):
	"""create a window for the advanced import"""

	def __init__(self, bibs={}, parent=None):
		"""Set some properties and call `initUI`

		Parameters:
			bibs: a dictionary containing the imported bibtex entries.
				Each element should be a dictionary containing at least
				a "bibpars" item, a dictionary with at least the keys
				["ID", "title", "author", "eprint", "doi"],
				and a boolean "exist" item.
			parent: the parent widget
		"""
		self.tableModel = None
		self.askCats = None
		self.acceptButton = None
		self.cancelButton = None
		self.abstractArea = None
		super(DailyArxivSelect, self).__init__(bibs, parent)

	def initUI(self):
		"""Initialize the widget content, with the buttons and labels"""
		self.setWindowTitle('ArXiv daily listing - results')

		self.currLayout.setSpacing(1)

		self.currLayout.addWidget(
			PBLabel("This is the list of elements found.\n"
				+ "Select the ones that you want to import:"))

		headers = ["eprint", "type", "title", "author", "primaryclass"]
		self.tableModel = PBImportedTableModel(self,
			self.bibs,
			headers + ["abstract"],
			idName="eprint")
		self.addFilterInput("Filter entries", gridPos=(1, 0))
		self.setProxyStuff(0, Qt.AscendingOrder)

		self.finalizeTable(gridPos=(2, 0, 1, 2))

		i = 3
		self.askCats = QCheckBox("Ask categories at the end?", self)
		self.askCats.toggle()
		self.currLayout.addWidget(self.askCats, i, 0)

		# OK button
		self.acceptButton = QPushButton('OK', self)
		self.acceptButton.clicked.connect(self.onOk)
		self.currLayout.addWidget(self.acceptButton, i + 1, 0)

		# cancel button
		self.cancelButton = QPushButton('Cancel', self)
		self.cancelButton.clicked.connect(self.onCancel)
		self.cancelButton.setAutoDefault(True)
		self.currLayout.addWidget(self.cancelButton, i + 2, 0)

		self.abstractArea = QTextEdit('Abstract', self)
		self.currLayout.addWidget(self.abstractArea, i + 3, 0, 4, 2)

	def cellClick(self, index):
		"""Click action

		Parameter:
			index: a `QModelIndex` instance
		"""
		if not index.isValid():
			return
		row = index.row()
		try:
			eprint = str(self.proxyModel.sibling(row, 0, index).data())
		except AttributeError:
			pBLogger.debug("Data not valid", exc_info=True)
			return
		if "already existing" in eprint:
			eprint = eprint.replace(" - already existing", "")
		if hasattr(self, "abstractFormulas") and \
				self.abstractFormulas is not None:
			a = self.abstractFormulas(self.parent(),
				self.bibs[eprint]["bibpars"]["abstract"],
				customEditor=self.abstractArea,
				statusMessages=False)
			a.doText()
		else:
			pBLogger.debug("self.abstractFormulas not present in "
				+ "DailyArxivSelect. Eprint: %s"%eprint)


class ExportForTexDialog(PBDialog):
	"""create a window for the "export for tex" function"""

	def __init__(self, parent=None):
		"""Simple extension of `PBDialog.__init__`"""
		super(ExportForTexDialog, self).__init__(parent)
		self.numTexFields = 1
		self.bibName = ""
		self.texFileNames = [""]
		self.texNames = []
		self.update = False
		self.remove = False
		self.result = False
		self.texButtons = None
		self.bibButton = None
		self.removeCheck = None
		self.updateCheck = None
		self.addTexButton = None
		self.acceptButton = None
		self.cancelButton = None
		self.grid = QGridLayout()
		self.setLayout(self.grid)
		self.initUI()

	def readForm(self):
		"""Read the form content"""
		self.update = self.updateCheck.isChecked()
		self.remove = self.removeCheck.isChecked()
		self.bibName = self.bibButton.text()
		if self.bibName == "Select file":
			self.bibName = ""
		self.texFileNames = []
		for button in self.texButtons:
			txt = button.text()
			if "[" in txt:
				try:
					txt = ast.literal_eval(txt)
				except ValueError:
					pBLogger.warning("Invalid text: %s"%s)
			self.texFileNames.append(txt)
		tmp = []
		for fn in self.texFileNames:
			if isinstance(fn, list):
				tmp += fn
			else:
				tmp.append(fn)
		self.texNames = [f for f in tmp if f != "Select file" and f != ""]

	def onAddTex(self):
		"""Add a new line for the texs"""
		self.numTexFields += 1
		self.readForm()
		for ix in range(self.numTexFields):
			try:
				a = self.texFileNames[ix]
			except IndexError:
				self.texFileNames.append("")
		self.cleanLayout()
		self.initUI()

	def onCancel(self):
		"""Reject the output (set self.result to False and close)"""
		self.result	= False
		self.close()

	def onOk(self):
		"""Accept the output (set self.result to True and close)"""
		self.result	= True
		self.readForm()
		self.close()

	def onAskBib(self):
		"""Accept the output (set self.result to True and close)"""
		outFName = askSaveFileName(self,
			title="Where do you want to export the entries?",
			filter="Bibtex (*.bib)")
		if outFName != "":
			self.bibButton.setText(outFName)

	def onAskTex(self, ix):
		"""Accept the output (set self.result to True and close)"""
		texFile = askFileNames(self,
			title="Which is/are the *.tex file(s) you want to compile?",
			filter="Latex (*.tex)")
		if texFile != "" and texFile != []:
			self.texButtons[ix].setText("%s"%texFile)

	def initUI(self):
		"""Create and fill the `QGridLayout`"""
		self.setWindowTitle('Export for tex file')
		self.grid.setSpacing(1)

		self.grid.addWidget(PBLabelRight("Bib file name:"), 0, 0, 1, 2)
		self.bibButton = QPushButton(
			"Select file" if self.bibName == "" else self.bibName,
			self)
		self.bibButton.clicked.connect(self.onAskBib)
		self.grid.addWidget(self.bibButton, 0, 2, 1, 2)

		self.texButtons = []
		for ix in range(self.numTexFields):
			self.grid.addWidget(
				PBLabelRight("Tex file name(s):"),
				ix+1, 0, 1, 2)
			self.texButtons.append(QPushButton(
				"Select file"
					if self.texFileNames[ix] == ""
					else "%s"%self.texFileNames[ix],
				self))
			self.texButtons[ix].clicked.connect(
				lambda s=False, p=ix: self.onAskTex(p))
			self.grid.addWidget(self.texButtons[ix], ix+1, 2, 1, 2)

		i = 2 + self.numTexFields
		self.addTexButton = QPushButton("Add more tex files", self)
		self.addTexButton.clicked.connect(self.onAddTex)
		self.grid.addWidget(self.addTexButton, i-1, 2)

		self.removeCheck = QCheckBox("Remove unused bibtexs?", self)
		if self.remove:
			self.removeCheck.setChecked(True)
		self.grid.addWidget(self.removeCheck, i, 0, 1, 2)
		self.updateCheck = QCheckBox("Update existing bibtexs?", self)
		if self.update:
			self.updateCheck.setChecked(True)
		self.grid.addWidget(self.updateCheck, i, 2, 1, 2)

		# OK button
		self.acceptButton = QPushButton('OK', self)
		self.acceptButton.clicked.connect(self.onOk)
		self.grid.addWidget(self.acceptButton, i+1, 1)

		# cancel button
		self.cancelButton = QPushButton('Cancel', self)
		self.cancelButton.clicked.connect(self.onCancel)
		self.cancelButton.setAutoDefault(True)
		self.grid.addWidget(self.cancelButton, i+1, 2)

		self.setGeometry(100, 100, 400, 25*(i+2))
		self.centerWindow()



"""Module that deals with importing info from the INSPIRE-HEP API.

This file is part of the physbiblio package.
"""
import traceback
import re
try:
	from physbiblio.errors import pBLogger
	from physbiblio.config import pbConfig
	from physbiblio.webimport.webInterf import WebInterf, physBiblioWeb
	from physbiblio.parseAccents import parse_accents_str
except ImportError:
	print("Could not find physbiblio and its modules!")
	print(traceback.format_exc())
	raise


class WebSearch(WebInterf):
	"""Subclass of WebInterf that can connect
	to INSPIRE-HEP to perform searches
	"""

	name = "inspire"
	description = "INSPIRE fetcher"
	url = pbConfig.inspireSearchBase
	urlRecord = pbConfig.inspireRecord
	urlArgs = {
		#"action_search": "Search",
		"sf": "year",
		"so": "a",
		"rg": "250",
		"sc": "0",
		"eb": "B",
		"of": "hx"
			# for bibtex format ---- hb for standard format,
			# for retrieving inspireid
		}

	def __init__(self):
		"""Initializes the class variables
		using the WebInterf constructor.

		Define additional specific parameters for the INSPIRE-HEP API.
		"""
		WebInterf.__init__(self)
		self.name = "inspire"
		self.description = "INSPIRE fetcher"
		self.url = pbConfig.inspireSearchBase
		self.urlRecord = pbConfig.inspireRecord
		self.urlArgs = {
			#"action_search": "Search",
			"sf": "year",
			"so": "a",
			"rg": "250",
			"sc": "0",
			"eb": "B",
			"of": "hx"
				# for bibtex format ---- hb for standard format,
				# for retrieving inspireid
			}

	def retrieveUrlFirst(self, string):
		"""Retrieves the first result
		from the content of the given web page.

		Parameters:
			string: the search string

		Output:
			returns the bibtex string obtained from the API
		"""
		self.urlArgs["p"] = string.replace(" ", "+")
		url = self.createUrl()
		pBLogger.info("Search '%s' -> %s"%(string, url))
		text = self.textFromUrl(url)
		try:
			i1 = text.find("<pre>")
			i2 = text.find("</pre>")
			if i1 > 0 and i2 > 0:
				bibtex = text[i1 + 5 : i2]
			else:
				bibtex = ""
			return parse_accents_str(bibtex)
		except Exception:
			pBLogger.exception("Impossible to get results")
			return ""

	def retrieveUrlAll(self, string):
		"""Retrieves all the result
		from the content of the given web page.

		Parameters:
			string: the search string

		Output:
			returns the bibtex string obtained from the API
		"""
		self.urlArgs["p"] = string.replace(" ", "+")
		url = self.createUrl()
		pBLogger.info("Search '%s' -> %s"%(string, url))
		text = self.textFromUrl(url)
		try:
			i1 = text.find("<pre>")
			i2 = text.rfind("</pre>")
			if i1 > 0 and i2 > 0:
				bibtex = text[i1 + 5 : i2]
			else:
				bibtex = ""
			return parse_accents_str(
				bibtex.replace("<pre>", "").replace("</pre>", ""))
		except Exception:
			pBLogger.exception("Impossible to get results")
			return ""

	def retrieveInspireID(self, string, number=None):
		"""Read the fetched content for a given entry
		to obtain its INSPIRE-HEP ID

		Parameters:
			string: the search string
			number (optional): the integer corresponding
				to the desired entry in the list,
				if more than one is present
		"""
		i = 0
		self.urlArgs["p"] = string.replace(" ", "+")
		self.urlArgs["of"] = "hb" #do not ask bibtex, but standard
		url = self.createUrl()
		self.urlArgs["of"] = "hx" #restore
		pBLogger.info("Search ID of %s -> %s"%(string, url))
		text = self.textFromUrl(url)
		if text is None:
			pBLogger.warning("An error occurred. Empty text obtained")
			return ""
		try:
			searchID = re.compile(
				'titlelink(.*)?(http|https)://inspirehep.net/record/([0-9]*)?">'
				)
			inspireID = ""
			for q in searchID.finditer(text):
				if len(q.group()) > 0:
					if number is None or i == number:
						inspireID = q.group(3)
						break
					else:
						i += 1
			pBLogger.info("Found: %s"%inspireID)
			return inspireID
		except Exception:
			pBLogger.exception("Impossible to get results")
			return ""



"""Module with the class that manages errors when the GUI is active.

This file is part of the physbiblio package.
"""
import sys
import logging
import traceback
from PySide2.QtWidgets import QMessageBox

if sys.version_info[0] < 3:
	from StringIO import StringIO
else:
	from io import StringIO

try:
	from physbiblio.errors import PBErrorManagerClass, pBLogger
except ImportError:
	print("Could not find physbiblio and its modules!")
	print(traceback.format_exc())


class ErrorStream(StringIO):
	"""Define a stream based on StringIO, which opens a `QMessageBox`
	when `write` is called
	"""

	def __init__(self, *args, **kwargs):
		"""The constructor, which passes the `*args, **kwargs` method
		to `StringIO.__init__`.
		"""
		StringIO.__init__(self, *args, **kwargs)
		self.priority = 1
		self.lastMBox = None

	def setPriority(self, prio):
		"""Set the priority level to be adopted
		when opening the `QMessageBox`.

		Parameter:
			prio: the priority level. 0, 1, 2+ correspond
				to `Information`, `Warning`, `Error`
		"""
		self.priority = prio

	def write(self, text):
		"""Override the `write` method of `StringIO`
		to show a `QMessageBox`,
		with icon according to the current priority.
		The priority is set to 1 after execution.

		Parameters:
			text: the text to display. "\n" is replaced with "<br>".

		Output:
			if text is empty or testing is False, return None
		"""
		if text.strip() == "":
			return
		text = text.replace('\n', '<br>')
		self.lastMBox = QMessageBox(QMessageBox.Information, "Information", "")
		self.lastMBox.setText(text)
		if self.priority == 0:
			self.lastMBox.setIcon(QMessageBox.Information)
			self.lastMBox.setWindowTitle("Information")
		elif self.priority > 1:
			self.lastMBox.setIcon(QMessageBox.Critical)
			self.lastMBox.setWindowTitle("Error")
		else:
			self.lastMBox.setIcon(QMessageBox.Warning)
			self.lastMBox.setWindowTitle("Warning")
		self.priority = 1
		self.lastMBox.exec_()


class PBErrorManagerClassGui(PBErrorManagerClass):
	"""Extends the error manager class to show
	gui messages through ErrorStream
	"""

	def __init__(self):
		"""Init the class, using PBErrorManagerClass.__init__ and
		the gui logger name,
		then add a new handler which uses ErrorStream
		"""
		PBErrorManagerClass.__init__(self, "physbiblioguilog")
		self.guiStream = ErrorStream()
		self.tempHandler(self.guiStream,
			level = logging.DEBUG,
			format = '%(message)s')

	def loggerPriority(self, prio):
		"""Define the priority level that must be used by
		the `ErrorStream` class in the first call of its `write` method.

		Parameter:
			prio: the priority level. 0, 1, 2+ correspond
				to Information, Warning, Error

		Output:
			the logger [so it can be used as
				`pBGUIErrorManager.loggerPriority(prio).info(message)`]
		"""
		self.guiStream.priority = prio
		return self.logger


pBGUIErrorManager = PBErrorManagerClassGui()
pBGUILogger = pBGUIErrorManager.logger



"""The package that contains all the GUI modules used by the PhysBiblio.

This file is part of the physbiblio package.
"""
__author__ = 'Stefano Gariazzo'
__email__ = "stefano.gariazzo@gmail.com"

import physbiblio
__version__ = physbiblio.__version__
__version_date__ = physbiblio.__version_date__

__all__ = [
	"basicDialogs",
	"bibWindows",
	"catWindows",
	"commonClasses",
	"dialogWindows",
	"errorManager",
	"expWindows",
	"inspireStatsGUI",
	"mainWindow",
	"marks",
	"profilesManager",
	"resourcesPyside2",
	"setuptests",
	"threadElements",
	]



#!/usr/bin/env python
"""Test file for the physbiblio.webimport subpackage.

This file is part of the physbiblio package.
"""
import sys
import datetime
import traceback
import os

if sys.version_info[0] < 3:
	import unittest2 as unittest
else:
	import unittest

try:
	from physbiblio.setuptests import *
	from physbiblio.webimport.webInterf import physBiblioWeb
	from physbiblio.webimport.inspireoai import get_journal_ref_xml
	from physbiblio.config import pbConfig
except ImportError:
	print("Could not find physbiblio and its modules!")
	raise
except Exception:
	print(traceback.format_exc())

@unittest.skipIf(skipTestsSettings.online, "Online tests")
class TestWebImportMethods(unittest.TestCase):
	"""Test the functions that import entries from the web.
	Should not fail if everything works fine.
	"""

	def test_methods_success(self):
		"""Test webimport with known results"""
		tests = {
			"arxiv": ["1507.08204",
				u'@Article{1507.08204,\n         title = ' \
				+ '"{Light sterile neutrinos}",\n          year = "2015",' \
				+ '\n archiveprefix = "arXiv",\n  primaryclass = ' \
				+ '"hep-ph",\n           doi = ' \
				+ '"10.1088/0954-3899/43/3/033001",\n      abstract = ' \
				+ '"{The theory and phenomenology of light sterile neutrinos' \
				+ ' at the eV mass scale is reviewed. The reactor, Gallium ' \
				+ 'and LSND anomalies are briefly described and interpreted ' \
				+ 'as indications of the existence of short-baseline ' \
				+ 'oscillations which require the existence of light sterile' \
				+ ' neutrinos. The global fits of short-baseline oscillation' \
				+ ' data in 3+1 and 3+2 schemes are discussed, together ' \
				+ 'with the implications for beta-decay and neutrinoless ' \
				+ 'double-beta decay. The cosmological effects of light ' \
				+ 'sterile neutrinos are briefly reviewed and the ' \
				+ 'implications of existing cosmological data are discussed.' \
				+ ' The review concludes with a summary of future ' \
				+ 'perspectives.}",\n         arxiv = "1507.08204",\n    ' \
				+ '   authors = "S. Gariazzo and C. Giunti and M. Laveder ' \
				+ 'and Y. F. Li and E. M. Zavanin",\n}'],
			"doi": ["10.1088/0954-3899/43/3/033001", '''@article{Gariazzo_2015,
	doi = {10.1088/0954-3899/43/3/033001},
	url = {''' + pbConfig.doiUrl + '''10.1088%2F0954-3899%2F43%2F3%2F033001},
	year = 2015,
	month = {mar},
	publisher = {{IOP} Publishing},
	volume = {43},
	number = {3},
	pages = {033001},
	author = {S Gariazzo and C Giunti and M Laveder and Y F Li and E M Zavanin},
	title = {Light sterile neutrinos},
	journal = {Journal of Physics G: Nuclear and Particle Physics}
}'''],
			"inspire": ["Gariazzo:2015rra", '''@article{Gariazzo:2015rra,
      author         = "Gariazzo, S. and Giunti, C. ''' \
+ '''and Laveder, M. and Li, Y. F.
                        and Zavanin, E. M.",
      title          = "{Light sterile neutrinos}",
      journal        = "J. Phys.",
      volume         = "G43",
      year           = "2016",
      pages          = "033001",
      doi            = "10.1088/0954-3899/43/3/033001",
      eprint         = "1507.08204",
      archivePrefix  = "arXiv",
      primaryClass   = "hep-ph",
      SLACcitation   = "%%CITATION = ARXIV:1507.08204;%%"
}'''],
			"inspireoai": ["1385583",
				{'doi': '10.1088/0954-3899/43/3/033001',
				'arxiv': '1507.08204', 'bibkey': 'Gariazzo:2015rra',
				'ads': '2015JPhG...43c3001G', 'journal': 'J.Phys.',
				'volume': 'G43', 'year': '2016', 'pages': '033001',
				'firstdate': '2015-07-29', 'pubdate': '2016-01-13',
				'author': 'Gariazzo, S. and Giunti, C. and Laveder, M. ' \
				+ 'and Li, Y.F. and Zavanin, E.M.', 'collaboration': None,
				'primaryclass': 'hep-ph', 'archiveprefix': 'arXiv',
				'eprint': '1507.08204', 'title': 'Light sterile neutrinos',
				'isbn': None, 'ENTRYTYPE': 'article', 'oldkeys': '',
				'bibtex': '@Article{Gariazzo:2015rra,\n        ' \
				+ 'author = "Gariazzo, S. and Giunti, C. and ' \
				+ 'Laveder, M. and Li, Y.F. and Zavanin, E.M.",\n  ' \
				+ '       title = "{Light sterile neutrinos}",\n   ' \
				+ '    journal = "J.Phys.",\n        volume = "G43",\n ' \
				+ '         year = "2016",\n         pages = "033001",\n ' \
				+ 'archiveprefix = "arXiv",\n  primaryclass = "hep-ph",\n ' \
				+ '       eprint = "1507.08204",\n           doi = ' \
				+ '"10.1088/0954-3899/43/3/033001",\n}\n\n',
				'id': '1385583',
				'link': '%s10.1088/0954-3899/43/3/033001'%pbConfig.doiUrl,
				'reportnumber': None}],
			"isbn": ["9780191523229", '''@book{9780191523229,
  Author = {Carlo Giunti, Chung W. Kim},
  Title = {Fundamentals of Neutrino Physics and Astrophysics},
  Publisher = {OUP Oxford},
  Year = {2007},
  Date = {2007-03-15},
  PageTotal = {},
  EAN = {9780191523229},
  URL = {https://www.ebook.de/de/product/21360145/carlo_giunti_''' \
+ '''chung_w_kim_fundamentals_of_neutrino_physics_and_astrophysics.html}
}'''],
			}
		for method, strings in tests.items():
			print(method)
			if method == "inspireoai":
				self.assertEqual(
					physBiblioWeb.webSearch[method].retrieveOAIData(
					strings[0]), strings[1])
			elif method == "arxiv":
				self.assertEqual(
					physBiblioWeb.webSearch[method].retrieveUrlFirst(
						strings[0], searchType="id").strip(),
					strings[1].strip())
				self.assertEqual(
					physBiblioWeb.webSearch[method].retrieveUrlAll(
						strings[0], searchType="id").strip(),
					strings[1].strip())
			else:
				self.assertEqual(
					physBiblioWeb.webSearch[method].retrieveUrlFirst(
						strings[0]).strip(),
					strings[1].strip())
				self.assertEqual(
					physBiblioWeb.webSearch[method].retrieveUrlAll(
						strings[0]).strip(),
					strings[1].strip())
		self.assertEqual(
			physBiblioWeb.webSearch["inspire"].retrieveInspireID(
				tests["inspire"][0]), u'1385583')

	def test_methods_insuccess(self):
		"""Test webimport using missing and/or invalid identifiers"""
		print(physBiblioWeb.webSearch.keys())
		tests = {
			"arxiv": ["1801.15000", ""],
			"doi": ["10.1088/9999-3899/43/a/033001", ""],
			"inspire": ["Gariazzo:2014rra", ""],
			"isbn": ["978019850871a", ""],
			}
		self.assertEqual(
			physBiblioWeb.webSearch["inspireoai"].retrieveOAIData("1110620"),
			{'doi': None, 'arxiv': None, 'bibkey': None, 'ads': None,
			'journal': None, 'volume': None, 'year': None, 'pages': None,
			'firstdate': None, 'pubdate': None, 'author': ' and Tauber, Jan',
			'collaboration': 'Planck',
			'title': 'Anisotropies of the Cosmic Background Radiation Field',
			'isbn': None, 'ENTRYTYPE': 'article', 'oldkeys': '',
			'bibtex': '@Article{,\n        author = " and Tauber, Jan",' \
			+ '\n collaboration = "Planck",\n         title = "' \
			+ '{Anisotropies of the Cosmic Background Radiation Field}",\n}' \
			+ '\n\n',
			'id': '1110620', 'archiveprefix': None, 'eprint': None,
			'link': None, 'primaryclass': None, 'reportnumber': None,})
		self.assertFalse(
			physBiblioWeb.webSearch["inspireoai"].retrieveOAIData("9999999"))
		for method, strings in tests.items():
			print(method)
			res=physBiblioWeb.webSearch[method].retrieveUrlFirst(strings[0])
			print(res)
			self.assertEqual(res.strip(), strings[1].strip())
			self.assertEqual(
				physBiblioWeb.webSearch[method].retrieveUrlAll(
				strings[0]).strip(), strings[1].strip())
		self.assertEqual(
			physBiblioWeb.webSearch["inspire"].retrieveInspireID(
			tests["inspire"][0]), "")

	def test_inspireoai_other(self):
		"""test auxiliary functions in inspireoai module"""
		self.maxDiff = None
		marcxmlRecord1 = physBiblioWeb.webSearch["inspireoai"].oai.getRecord(
			metadataPrefix='marcxml',
			identifier="oai:inspirehep.net:1385583")[1]
		marcxmlRecord2 = physBiblioWeb.webSearch["inspireoai"].oai.getRecord(
			metadataPrefix='marcxml',
			identifier="oai:inspirehep.net:1414175")[1]
		self.assertEqual(get_journal_ref_xml(marcxmlRecord1),
			(['J.Phys.'], ['G43'], ['2016'], ['033001'],
			[None], [None], [None], [None]))
		self.assertEqual(get_journal_ref_xml(marcxmlRecord2),
			([None], [None], ['2017'], ['469-475'], [None],
			[None], [None], ['C15-08-20']))
		dict1a = physBiblioWeb.webSearch["inspireoai"].readRecord(
			marcxmlRecord1)
		dict1b = physBiblioWeb.webSearch["inspireoai"].readRecord(
			marcxmlRecord1, readConferenceTitle=True)
		self.assertEqual(dict1a, dict1b)
		self.assertEqual(dict1a,
			{'doi': '10.1088/0954-3899/43/3/033001',
			'arxiv': '1507.08204', 'bibkey': 'Gariazzo:2015rra',
			'ads': '2015JPhG...43c3001G', 'journal': 'J.Phys.',
			'volume': 'G43', 'year': '2016', 'pages': '033001',
			'firstdate': '2015-07-29', 'pubdate': '2016-01-13',
			'author': 'Gariazzo, S. and Giunti, C. and Laveder, M. and ' \
			+ 'Li, Y.F. and Zavanin, E.M.',
			'collaboration': None, 'primaryclass': 'hep-ph',
			'archiveprefix': 'arXiv', 'eprint': '1507.08204',
			'reportnumber': None, 'title': 'Light sterile neutrinos',
			'isbn': None, 'ENTRYTYPE': 'article', 'oldkeys': '',
			'link': '%s10.1088/0954-3899/43/3/033001'%pbConfig.doiUrl,
			'bibtex': '@Article{Gariazzo:2015rra,\n        ' \
			+ 'author = "Gariazzo, S. and Giunti, C. and Laveder, M. ' \
			+ 'and Li, Y.F. and Zavanin, E.M.",\n         title = "' \
			+ '{Light sterile neutrinos}",\n       journal = ' \
			+ '"J.Phys.",\n        volume = "G43",\n          year = ' \
			+ '"2016",\n         pages = "033001",\n archiveprefix = ' \
			+ '"arXiv",\n  primaryclass = "hep-ph",\n        eprint = ' \
			+ '"1507.08204",\n           doi = ' \
			+ '"10.1088/0954-3899/43/3/033001",\n}\n\n'})
		dict2 = physBiblioWeb.webSearch["inspireoai"].readRecord(
			marcxmlRecord2, readConferenceTitle=True)
		self.assertEqual(dict2,
			{'doi': '10.1142/9789813224568_0076', 'arxiv': '1601.01475',
			'bibkey': 'Gariazzo:2016ehl', 'ads': None, 'journal': None,
			'volume': None, 'year': '2017', 'pages': '469-475',
			'firstdate': '2016-01-07', 'pubdate': '2017',
			'author': 'Gariazzo, Stefano', 'collaboration': None,
			'primaryclass': 'astro-ph.CO', 'archiveprefix': 'arXiv',
			'eprint': '1601.01475', 'reportnumber': None,
			'title': 'Light Sterile Neutrinos In Cosmology',
			'isbn': None, 'booktitle': 'Proceedings, 17th Lomonosov ' \
			+ 'Conference on Elementary Particle Physics: Moscow, ' \
			+ 'Russia, August 20-26, 2015', 'ENTRYTYPE': 'inproceedings',
			'oldkeys': '',
			'link': '%s10.1142/9789813224568_0076'%pbConfig.doiUrl,
			'bibtex': '@Inproceedings{Gariazzo:2016ehl,\n        ' \
			+ 'author = "Gariazzo, Stefano",\n         ' \
			+ 'title = "{Light Sterile Neutrinos In Cosmology}",' \
			+ '\n     booktitle = "{Proceedings, 17th Lomonosov ' \
			+ 'Conference on Elementary Particle Physics: Moscow, ' \
			+ 'Russia, August 20-26, 2015}",\n          year = ' \
			+ '"2017",\n         pages = "469-475",\n archiveprefix = ' \
			+ '"arXiv",\n  primaryclass = "astro-ph.CO",\n        ' \
			+ 'eprint = "1601.01475",\n           doi = ' \
			+ '"10.1142/9789813224568_0076",\n}\n\n'})
		dict1a["id"] = "1385583"
		dict2["id"] = "1414175"
		bibtex1 = '@article{abc,\nauthor="me",}'
		bibtex2 = '@article{abc,'
		self.assertEqual(physBiblioWeb.webSearch["inspireoai"].updateBibtex(
			dict2, bibtex1), (False, bibtex1))
		self.assertEqual(physBiblioWeb.webSearch["inspireoai"].updateBibtex(
			dict1a, bibtex2), (False, bibtex2))
		self.assertEqual(physBiblioWeb.webSearch["inspireoai"].updateBibtex(
			dict1a, bibtex1),
			(True, '@Article{abc,\n        author = "me",\n       ' \
			+ 'journal = "J.Phys.",\n        volume = "G43",\n          ' \
			+ 'year = "2016",\n         pages = "033001",\n           ' \
			+ 'doi = "10.1088/0954-3899/43/3/033001",\n}\n\n'))

	@unittest.skipIf(skipTestsSettings.oai, "Online tests with OAI")
	def test_inspireoai(self):
		"""test retrieve daily data from inspireOAI"""
		date1 = (datetime.date.today() - datetime.timedelta(1)
			).strftime("%Y-%m-%d")
		date2 = datetime.date.today().strftime("%Y-%m-%d")
		yren, monen, dayen = date1.split('-')
		yrst, monst, dayst = date2.split('-')
		date1 = datetime.datetime(int(yren), int(monen), int(dayen))
		date2 = datetime.datetime(int(yrst), int(monst), int(dayst))
		result = physBiblioWeb.webSearch["inspireoai"].retrieveOAIUpdates(
			date1, date2)
		self.assertEqual(type(result), list)
		print(len(result), result[0])


class TestWebImportOffline(unittest.TestCase):
	"""Offline test for some functions of the webImport package."""

	def test_arxivYear(self):
		"""test the arxivDaily method in the arxiv module"""
		self.assertEqual(physBiblioWeb.webSearch["arxiv"].getYear("abc"), None)
		self.assertEqual(physBiblioWeb.webSearch["arxiv"].getYear("0123.345"),
			None)
		self.assertEqual(physBiblioWeb.webSearch["arxiv"].getYear(
			"hep-ph/9900000"), "1999")
		self.assertEqual(physBiblioWeb.webSearch["arxiv"].getYear(
			"hep-ph/990000"), None)
		self.assertEqual(physBiblioWeb.webSearch["arxiv"].getYear("1234.5678"),
			"2012")
		self.assertEqual(physBiblioWeb.webSearch["arxiv"].getYear(
			"1678.56789"), "2016")

	def test_arxivDaily(self):
		"""test the arxivDaily method in the arxiv module"""
		self.assertFalse(physBiblioWeb.webSearch["arxiv"].arxivDaily(
			"missing"))
		self.assertFalse(physBiblioWeb.webSearch["arxiv"].arxivDaily(
			"physics.missing"))
		content_example='''<?xml version="1.0" encoding="UTF-8"?>

<rdf:RDF
 xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#"
 xmlns="http://purl.org/rss/1.0/"
 xmlns:content="http://purl.org/rss/1.0/modules/content/"
 xmlns:taxo="http://purl.org/rss/1.0/modules/taxonomy/"
 xmlns:dc="http://purl.org/dc/elements/1.1/"
 xmlns:syn="http://purl.org/rss/1.0/modules/syndication/"
 xmlns:admin="http://webns.net/mvcb/"
>

<channel rdf:about="http://arxiv.org/">
<title>hep-ex updates on arXiv.org</title>
<link>http://arxiv.org/</link>
<description rdf:parseType="Literal">High Energy Physics - ''' \
+ '''Experiment (hep-ex) updates on the arXiv.org e-print archive</description>
<dc:language>en-us</dc:language>
<dc:date>2018-05-23T20:30:00-05:00</dc:date>
<dc:publisher>www-admin@arxiv.org</dc:publisher>
<dc:subject>High Energy Physics - Experiment</dc:subject>
<syn:updateBase>1901-01-01T00:00+00:00</syn:updateBase>
<syn:updateFrequency>1</syn:updateFrequency>
<syn:updatePeriod>daily</syn:updatePeriod>
<items>
 <rdf:Seq>
  <rdf:li rdf:resource="http://arxiv.org/abs/1805.08875" />
  <rdf:li rdf:resource="http://arxiv.org/abs/1805.09166" />
  <rdf:li rdf:resource="http://arxiv.org/abs/1805.06430" />
 </rdf:Seq>
</items>
<image rdf:resource="http://arxiv.org/icons/sfx.gif" />
</channel>
<image rdf:about="http://arxiv.org/icons/sfx.gif">
<title>arXiv.org</title>
<url>http://arxiv.org/icons/sfx.gif</url>
<link>http://arxiv.org/</link>
</image>
<item rdf:about="http://arxiv.org/abs/1805.08875">
<title>Measurement of Transverse Single Spin Asymmetries. ''' \
+ '''(arXiv:1805.08875v1 [hep-ex])</title>
<link>http://arxiv.org/abs/1805.08875</link>
<description rdf:parseType="Literal">&lt;p&gt;In 2015 the first ''' \
+ '''collisions between polarized protons and nuclei occurred at
the Relativistic Heavy Ion Collider (RHIC), at a center-of-mass energy of
$\sqrt{s_{NN}}=200$ GeV. Comparisons between spin asymmetries and
cross-sections in $p+p$ production to those in $p+A$ production provide insight
into nuclear structure, namely nuclear modification factors, nuclear dependence
of spin asymmetries, and comparison to models with saturation effects. The
transverse single-spin asymmetry, $A_{N}$, has been measured in $\pi^{0}$
production in the STAR Forward Meson Spectrometer (FMS), an electromagnetic
calorimeter covering a forward psuedorapidity range ''' \
+ '''of $2.6&amp;lt;\eta&amp;lt;4$. Within
this kinematic range, STAR has previously reported the persistence of large
$\pi^0$ asymmetries with unexpected dependences on $p_T$ and event topology in
$p+p$ collisions. This talk will compare these dependences to those in $p+A$
production.
&lt;/p&gt;
</description>
<dc:creator> &lt;a href=&quot;http://arxiv.org/find/hep-ex/1/''' \
+ '''au:+Dilks_C/0/1/0/all/0/1&quot;&gt;Christopher Dilks&lt;/a&gt; ''' \
+ '''(STAR Collaboration)</dc:creator>
</item>
<item rdf:about="http://arxiv.org/abs/1805.09166">
<title>Recent Top quark results from the Tevatron. ''' \
+ '''(arXiv:1805.09166v1 [hep-ex] CROSS LISTED)</title>
<link>http://arxiv.org/abs/1805.09166</link>
<description rdf:parseType="Literal">&lt;p&gt;We present recent measurements
&lt;/p&gt;
</description>
<dc:creator> &lt;a href=&quot;http://arxiv.org/find/''' \
+ '''hep-ex/1/au:+Tuchming_B/0/1/0/all/0/1&quot;&gt;''' \
+ '''Boris Tuchming&lt;/a&gt;</dc:creator>
</item>
<item rdf:about="http://arxiv.org/abs/1805.06430">
<title>New Perspective on Hybrid Mesons. (arXiv:1805.06430v2 ''' \
+ '''[nucl-th] UPDATED)</title>
<link>http://arxiv.org/abs/1805.06430</link>
<description rdf:parseType="Literal">&lt;p&gt;It is thought that ''' \
+ '''strong interactions within the Standard Model can generate
bound-states in which non-Abelian gauge-bosons play a dual role, serving both
as force and matter fields.
&lt;/p&gt;
</description>
<dc:creator> &lt;a href=&quot;http://arxiv.org/find/''' \
+ '''nucl-th/1/au:+Xu_S/0/1/0/all/0/''' \
+ '''1&quot;&gt;Shu-Sheng Xu&lt;/a&gt;, &lt;a ''' \
+ '''href=&quot;http://arxiv.org/find/''' \
+ '''nucl-th/1/au:+Cui_Z/0/1/0/all/''' \
+ '''0/1&quot;&gt;Zhu-Fang Cui&lt;/a&gt;, &lt;a ''' \
+ '''href=&quot;http://arxiv.org/find/nucl-th/1/au:+''' \
+ '''Chang_L/0/1/0/all/0/1&quot;&gt;Lei Chang&lt;/''' \
+ '''a&gt;, &lt;a href=&quot;http://arxiv.org/find/nucl-th/1/au:+''' \
+ '''Papavassiliou_J/0/1/0/all/0/1&quot;&gt;Joannis ''' \
+ '''Papavassiliou&lt;/a&gt;, &lt;a href=&quot;http://arxiv.org/find/''' \
+ '''nucl-th/1/au:+Roberts_C/0/1/0/all/0/1&quot;''' \
+ '''&gt;Craig D. Roberts&lt;/a&gt;, &lt;a ''' \
+ '''href=&quot;http://arxiv.org/find/nucl-th/''' \
+ '''1/au:+Zong_H/0/1/0/all/0/1&quot;&gt;Hong-Shi Zong''' \
+ '''&lt;/a&gt;</dc:creator>
</item>
</rdf:RDF>'''
		with patch('physbiblio.webimport.webInterf.WebInterf.textFromUrl',
				return_value=content_example, autospec=True) as _fromUrl,\
				patch.dict(pbConfig.params, {"maxAuthorNames": 3},
					clear=False):
			result = physBiblioWeb.webSearch["arxiv"].arxivDaily("hep-ex")
			_fromUrl.assert_called_once_with(physBiblioWeb.webSearch["arxiv"],
				"https://export.arxiv.org/rss/hep-ex")
			self.assertEqual(result[0],
				{'primaryclass': u'hep-ex', 'author': u'Christopher Dilks',
				'title': u'Measurement of Transverse Single ' \
				+ 'Spin Asymmetries.',
				'abstract': u'In 2015 the first collisions between ' \
				+ 'polarized protons and nuclei occurred at the ' \
				+ 'Relativistic Heavy Ion Collider (RHIC), at a ' \
				+ 'center-of-mass energy of $\\sqrt{s_{NN}}=200$ GeV. ' \
				+ 'Comparisons between spin asymmetries and cross-sections ' \
				+ 'in $p+p$ production to those in $p+A$ production ' \
				+ 'provide insight into nuclear structure, namely ' \
				+ 'nuclear modification factors, nuclear dependence ' \
				+ 'of spin asymmetries, and comparison to models ' \
				+ 'with saturation effects. The transverse single-spin ' \
				+ 'asymmetry, $A_{N}$, has been measured in $\\pi^{0}$ ' \
				+ 'production in the STAR Forward Meson Spectrometer ' \
				+ '(FMS), an electromagnetic calorimeter covering a ' \
				+ 'forward psuedorapidity range of $2.6&lt;\\eta&lt;4$. ' \
				+ 'Within this kinematic range, STAR has previously ' \
				+ 'reported the persistence of large $\\pi^0$ asymmetries ' \
				+ 'with unexpected dependences on $p_T$ and event topology ' \
				+ 'in $p+p$ collisions. This talk will compare ' \
				+ 'these dependences to those in $p+A$ production. ',
				'cross': False, 'version': u'1805.08875v1',
				'eprint': u'1805.08875',
				'authors': [u'Christopher Dilks'], 'replacement': False})
			self.assertEqual(result[1],
				{'primaryclass': u'hep-ex', 'author': u'Boris Tuchming',
				'title': u'Recent Top quark results from the Tevatron.',
				'abstract': u'We present recent measurements ',
				'cross': True, 'version': u'1805.09166v1',
				'eprint': u'1805.09166', 'authors': [u'Boris Tuchming'],
				'replacement': False})
			self.assertEqual(result[2],
				{'primaryclass': u'nucl-th',
				'author': u'Shu-Sheng Xu and Zhu-Fang Cui and ' \
				+ 'Lei Chang and others',
				'title': u'New Perspective on Hybrid Mesons.',
				'abstract': u'It is thought that strong interactions ' \
				+ 'within the Standard Model can generate bound-states ' \
				+ 'in which non-Abelian gauge-bosons play a dual role, ' \
				+ 'serving both as force and matter fields. ',
				'cross': True, 'version': u'1805.06430v2',
				'eprint': u'1805.06430',
				'authors': [u'Shu-Sheng Xu', u'Zhu-Fang Cui', u'Lei Chang',
					u'Joannis Papavassiliou', u'Craig D. Roberts',
					u'Hong-Shi Zong'],
				'replacement': True})


if __name__=='__main__':
	unittest.main()
